#ifndef KNN_INC
#define KNN_INC

#include "Machine.h"
#include "DataSet.h"

/** This machine implements the K-neirest-neighbors (KNN) algorithm.
    Given a dataset (in the constructor), the #forward# method returns
    for a given input the average of the outputs of the K neirest examples
    (in the input space, using the Euclidean distance). As a side effect,
    the machine also keep the table of distances of the K-neirest-neighbors.

    @author Samy Bengio (bengio@idiap.ch)
*/
class KNN : public Machine
{
  public:

    /// The number of neirest neighbors. Controls the capacity of the machine
    int K;
    /// For each neirest neighbor, keeps its distance to the current input
    real* distances;
    /// For each neirest neighbor, keeps its index in the dataset
    int* indices;
    /// The dataset that contains the potential neighbors
    DataSet* data;

    /// the indices of the training examples
    int *real_examples;
    int n_real_examples;

    ///
    KNN(DataSet* data_,int K_);

    virtual void forward(List *inputs);
    virtual void init();
    virtual void reset();

    /// change the value of K
    virtual void setK(int K_);

    virtual ~KNN();
};

#endif
