#include "MLP.h"

MLP::MLP(int n_inputs_, int n_hidden_, int n_outputs_)
{
  n_inputs = n_inputs_;
  n_hidden = n_hidden_;
  n_outputs = n_outputs_;
  bool lo = true;
  addOption("linear outputs", sizeof(bool), &is_linear_outputs, &lo, "linear outputs");
  bool itoo = false;
  addOption("inputs to outputs", sizeof(bool), &inputs_to_outputs, &itoo, "connections from inputs to outputs");
  is_softmax_outputs = false;
  addOption("softmax outputs", sizeof(bool), &is_softmax_outputs, &is_softmax_outputs, "softmax outputs");
  is_sigmoid_outputs = false;
  addOption("sigmoid outputs", sizeof(bool), &is_sigmoid_outputs, &is_sigmoid_outputs, "sigmoid outputs");
  real wdf = 0;
  addOption("weight decay", sizeof(real), &weight_decay, &wdf, "weight decay");
}

void MLP::init()
{
  add_layer = NULL;
  sum_layer = NULL;
  if (n_hidden>0) {
    hidden_layer = new Linear(n_inputs,n_hidden);
    hidden_layer->setROption("weight decay",weight_decay);
    hidden_layer->init();
    addMachine(hidden_layer);

    if (inputs_to_outputs) {
      add_layer = new Linear(n_inputs,n_outputs);
      add_layer->setROption("weight decay",weight_decay);
      add_layer->init();
      addMachine(add_layer);
    }

    hidden_tanh_layer = new Tanh(n_hidden);
    hidden_tanh_layer->init();
    addLayer();
    addMachine(hidden_tanh_layer);
    connectOn(hidden_layer);

    addLayer();
    outputs_layer = new Linear(n_hidden,n_outputs);
    outputs_layer->setROption("weight decay",weight_decay);
    outputs_layer->init();
    addMachine(outputs_layer);
    connectOn(hidden_tanh_layer);

    if (inputs_to_outputs) {
      sum_layer = new SumMachine(n_outputs,2);
      sum_layer->init();
      addLayer();
      addMachine(sum_layer);
      connectOn(add_layer);
      connectOn(outputs_layer);
    }
  } else {
    outputs_layer = new Linear(n_inputs,n_outputs);
    outputs_layer->setROption("weight decay",weight_decay);
    outputs_layer->init();
    addFCL(outputs_layer);
  }

  if(is_softmax_outputs)
  {
    outputs_softmax_layer = new Softmax(n_outputs);
    outputs_softmax_layer->init();
    addFCL(outputs_softmax_layer);
  }
  else
  {
    if(is_sigmoid_outputs)
    {
      outputs_sigmoid_layer = new Sigmoid(n_outputs);
      outputs_sigmoid_layer->init();
      addFCL(outputs_sigmoid_layer);
    }
    else
    {
      if (!is_linear_outputs) {
        outputs_tanh_layer = new Tanh(n_outputs);
        outputs_tanh_layer->init();
        addFCL(outputs_tanh_layer);
      }
    }
  }
  allocateMemory();
}

MLP::~MLP()
{
  if (n_hidden>0) {
    delete hidden_layer;
    delete hidden_tanh_layer;
  }
  delete outputs_layer;

  if(is_softmax_outputs)
    delete outputs_softmax_layer;
  else
  {
    if (!is_linear_outputs)
      delete outputs_tanh_layer;
  }

  if (add_layer)
    delete add_layer;
  if (inputs_to_outputs)
    delete sum_layer;
}

