#include "ParzenMachine.h"

ParzenMachine::ParzenMachine(DataSet* data_,real var_)
{
  data = data_;
  setVar(var_);
  n_outputs = data->n_targets;
  n_inputs = data->n_inputs;
  addToList(&outputs, n_outputs, (real *)xalloc(sizeof(real)*n_outputs));
  n_real_examples = 0;
  real_examples = NULL;
}

void ParzenMachine::setVar(real var_)
{
  var = var_;
}

void ParzenMachine::init()
{
  // do it your self in the main
  //reset();
}

void ParzenMachine::reset()
{
  data->setExample(0);
  n_real_examples = data->n_examples;
  real_examples = (int*)xrealloc(real_examples,n_real_examples*sizeof(int));
  for (int i=0;i<data->n_examples;i++) {
    real_examples[i] = data->selected_examples[i];
  }
}

void ParzenMachine::forward(List* inputs)
{
  // keep current example in order to restore at the end
  int current_example = data->current_example;

  real* out = (real*)outputs->ptr;
  denominator = 0.;
  for (int j=0;j<n_outputs;j++)
    *out++ = 0;
  int *i_ptr = real_examples;
  for (int i=0;i<n_real_examples;i++) {
    data->setRealExample(*i_ptr++);
    real* in = (real*)inputs->ptr;
    real* in_i = (real*)data->inputs->ptr;
    real dist = 0;
    for (int j=0;j<n_inputs;j++) {
      real z = *in++ - *in_i++;
      dist += z*z;
    }
    real e = exp(-dist / (2.*var));
    denominator += e;
    out = (real*)outputs->ptr;
    real* targ_i = (real*)data->targets;
    for (int j=0;j<n_outputs;j++) {
      *out++ += *targ_i++ * e;
    }
  }

  out = (real*)outputs->ptr;
  for (int j=0;j<n_outputs;j++)
    *out++ /= denominator;

  // restore current_example
  data->setRealExample(current_example);
}

ParzenMachine::~ParzenMachine()
{
  freeList(&outputs,true);
  free(real_examples);
}
