#include "RBF.h"

RBF::RBF(int n_inputs_, int n_hidden_, int n_outputs_, bool is_linear_outputs_, bool inputs_to_outputs_,EMTrainer* kmeans_trainer)
{
  n_hidden = n_hidden_;
  is_linear_outputs = is_linear_outputs_;
  inputs_to_outputs = inputs_to_outputs_;
  add_layer = NULL;
  sum_layer = NULL;
  if (n_hidden>0) {
    log_rbf_layer= new LogRBF(n_inputs_,n_hidden,kmeans_trainer);
    log_rbf_layer->init();
    addMachine(log_rbf_layer);

    if (inputs_to_outputs_) {
      add_layer = new Linear(n_inputs_,n_outputs_);
      add_layer->init();
      addMachine(add_layer);
    }

    exp_layer = new Exp(n_hidden);
    exp_layer->init();
    addLayer();
    addMachine(exp_layer);
    connectOn(log_rbf_layer);

    addLayer();
    outputs_layer = new Linear(n_hidden,n_outputs_);
    outputs_layer->init();
    addMachine(outputs_layer);
    connectOn(exp_layer);

    if (inputs_to_outputs_) {
      sum_layer = new SumMachine(n_outputs_,2);
      sum_layer->init();
      addLayer();
      addMachine(sum_layer);
      connectOn(add_layer);
      connectOn(outputs_layer);
    }
  } else {
    outputs_layer = new Linear(n_inputs_,n_outputs_);
    outputs_layer->init();
    addFCL(outputs_layer);
  }
  if (!is_linear_outputs) {
    outputs_tanh_layer = new Tanh(n_outputs_);
    outputs_tanh_layer->init();
    addFCL(outputs_tanh_layer);
  }
}

RBF::~RBF()
{
  if (n_hidden>0) {
    delete log_rbf_layer;
    delete exp_layer;
  }
  delete outputs_layer;
  if (!is_linear_outputs)
    delete outputs_tanh_layer;
  if (add_layer)
    delete add_layer;
  if (inputs_to_outputs)
    delete sum_layer;
}

