
#include "AdaBoostM1.h"

AdaBoostM1::AdaBoostM1( BoostMachine* machine_, DataSet* data_ )
  : Boost( machine_, data_ )
{
 beta = ( real* )xalloc( boostmachine->n_trainers*sizeof( real ));
 subset = ( int* )xalloc( data->n_examples * sizeof( int ));

 for( int i = 0; i < boostmachine->n_trainers; i++)
   beta[ i ] = 1;

 tabclasses[ 0 ] = 0;
 tabclasses[ 1 ] = 1;

 //in the case where we have 2 classes, we
 //want to find which classes are used.

 if( data->n_targets == 1 )
 {
   int n_set = 0;
   for(int i = 0; i < data->n_examples; i++)
   {
     data->setExample(i);
     
     bool flag = false;
     for(int k = 0; k < n_set; k++)
     {
       if(((real *)data->targets)[0] == tabclasses[k])
         flag = true;
     }
     
     if(!flag)
     {
       if(n_set == 2)
         error("TwoClassMeasurer: you have more than two classes");
       
       tabclasses[n_set++] = ((real *)data->targets)[0];
     }
   }
 }
}

AdaBoostM1::~AdaBoostM1()
{
  free( subset ); 
  free( beta );
}

void AdaBoostM1::train( List* measurers )
{
  message( "AdaBoost.M1: Training Boosty..." );

  real sum1 ;
  
  for( int trainer = 0; trainer < boostmachine->n_trainers; trainer++ )
  {

    sum1 = 0;
    /** 
     * get a Weaklearning rule and provide it with distribution D_t 
     */

    if ( !weighting )
    {
      boostmachine->prob_inter[ 0 ] = boostmachine->prob_dist[ 0 ];
      
      //first, calculation of the probability intervals
      
      for( int i = 1; i < data->n_examples; i++ )
        boostmachine->prob_inter[ i ] = boostmachine->prob_inter[ i - 1 ] + boostmachine->prob_dist[ i ] ;

      message(" >>> sampling <<< ");
      
      //once the probability intervals have been calculated, we can go 
      //on to do non-uniforme sampling, using a binary search on the probabilty
      //distribution function.
      
      
      for( int example = 0 ; example < data->n_examples; example++ )
      {
        
        real hasard = bounded_uniform(0,1);
        
        int left = 0;
        int right = data->n_examples - 1;
        int center;
      
        //in the case where we want example 0;
        if( hasard < boostmachine->prob_inter[ 0 ]){
          subset[ example ] = 0;
          continue;
        }
          

        while( (left + 1) != right )
        {
          center = ( right + left ) / 2;
          if( hasard < boostmachine->prob_inter[ center ] )
            right = center;
          else
            left = center;
        }
        subset[ example ] = right;

        //cout << right << ' ' << boostmachine->prob_dist[ right ] << ' ' << hasard << endl;
        
      }
    }
    else
    {
      
      message(" >>> weighting <<< ");
      
      GMTrainer* tmp_trainer = ( GMTrainer* ) boostmachine->trainers[ trainer ];
      WeightedMseCriterion* tmp_criterion = ( WeightedMseCriterion* ) ( tmp_trainer->criterion );

      //lets find the max of the weights 
      real max = boostmachine->prob_dist[ 0 ];
      
      for( int i = 1; i < data->n_examples; i++)
        if( max < boostmachine->prob_dist[ i ] )
          max = boostmachine->prob_dist[ i ];


      for( int i = 0; i < data->n_examples; i++ )
        tmp_criterion->weights[ i ] = boostmachine->prob_dist[ i ] / max ;
      
      for( int i = 0; i < data->n_examples; i++ )
        subset[ i ] = i;
    }
    
    
    /**
     * train this weaklearner with the given distribution 
     * and with the measurer associated with this trainer, given
     * by the user
     */
    

    //push the distribution in the trainer
    data->pushSubset( subset, data->n_examples );
    
    boostmachine->trainers[ trainer ]->machine->reset();
    boostmachine->trainers[ trainer ]->train( boostmachine->trainers_measurers[ trainer ]);
    
    data->popSubset();
    
    /**
     * calculate the error of the weaklearner on the distribution whole data set.
     *
     * Here we must differentiate between two cases. The two class and multi 
     * (... more than two ... ) class problems. If we have two classes, than
     * the actual target is not a vector of size two, but a real who takes
     * two particular values tabclasses[0] and tabclasses[1]. 
     * In a multi-class ( > 2) problem, the target is a vector
     * in which the index with the maximal value is the target class. 
     *
     *
     *
     */

    // and now we are ready to test on the whole dataset.
    real error = 0;

    for( int i = 0; i < data->n_examples; i++ )
      subset[ i ] = i;
    
    data->pushSubset( subset, data->n_examples );
    
    for( int example = 0; example < data->n_examples; example++ ){
      
      //these long lines aren't necessary since every Trainer has a pointer
      //to the same dataSet: a setExample in one of them sets is valid for 
      //all the other ones.
      boostmachine->trainers[ trainer ]->data->setExample( example );
      boostmachine->trainers[ trainer ]->machine->forward( boostmachine->trainers[ trainer ]->data->inputs );
      
      //the predicted class by the trainer

      real* trainer_output = ( real* ) boostmachine->trainers[ trainer ]->machine->outputs->ptr;
      
      real predicted_class = tabclasses[ 0 ];
	  
      if( data->n_targets == 1 )
      {
        if( fabs(*trainer_output - tabclasses[0]) > fabs(*trainer_output - tabclasses[1]))
          predicted_class = tabclasses[ 1 ];
      }
      else
      {
        predicted_class = 0;
        real max_val = trainer_output[ 0 ]; 
	  
        for( int output = 0; output < boostmachine->trainers[ trainer ]->machine->n_outputs; output++ )
          if( max_val < trainer_output[ output ] )
          {
            max_val = trainer_output[ output ];
            predicted_class = output;
          }
      }     
      
      //the examples' target.
      real* targets = ( real* ) boostmachine->trainers[ trainer ]->data->targets;

      real target_class;
      
      if( data->n_targets == 1 )
      {
        target_class = *targets;
      }
      else
      {
        target_class = 0;
        real max_val = targets[ 0 ];
        
        for( int target = 0; target < data->n_targets; target++ )
          if( max_val < targets[ target ] )
          {
            max_val = targets[ target ];
            target_class = target;
          }
      }
      
      
      // the ultimate test!
      if( predicted_class != target_class)
      {
        sum1++;
        error += boostmachine->prob_dist[ example ];
      }
    }
    
    message("misclassified : %f\n",sum1);
    message("error : %f\n",error);
      
      
    //if the error is more than one half, if the weaklearning rule is
    //actually worst than guessing, then set T = t - 1 and abort loop
    

    if( error >= 0.5 )
      {
        message( "Weaklearner too weak. Aborting Boosting.");
        trainer = boostmachine->n_trainers - 1;
        data->popSubset();
        continue;
      }

    /**
     * set beta
     */

    if( error == 0 )
      beta[ trainer ] = 1.0/ ( real ) RAND_MAX;
    else
      beta[ trainer ] = error / ( 1.0 - error );

    message("beta[%d] = %f\n",trainer,beta[trainer]);
      
    /** 
     * update the distribution 
     */
    
    for( int example = 0; example < data->n_examples; example++ )
    {
      
      boostmachine->trainers[ trainer ]->data->setExample( example );
      boostmachine->trainers[ trainer ]->machine->forward( boostmachine->trainers[ trainer ]->data->inputs );
      
      //the predicted class by the trainer
      real* trainer_output = ( real* ) boostmachine->trainers[ trainer ]->machine->outputs->ptr;
      
      real predicted_class = tabclasses[ 0 ];
      
      if( boostmachine->trainers[ trainer ]->machine->n_outputs == 1)
      {
        if( fabs(*trainer_output - tabclasses[0]) > fabs(*trainer_output - tabclasses[1]))
          predicted_class = tabclasses[ 1 ];
      }
      else
      {
        predicted_class = 0;
        real max_val = trainer_output[ 0 ];
	  
        for( int output = 0; output < boostmachine->trainers[ trainer ]->machine->n_outputs; output++ )
          if( max_val < trainer_output[ output ] )
          {
            max_val = trainer_output[ output ];
            predicted_class = output;
          }
      }

      //the examples' target.
      real* targets = ( real* )boostmachine->trainers[ trainer ]->data->targets;
      
      real target_class;

      if( boostmachine->trainers[ trainer ]->machine->n_outputs == 1)
      {
        target_class = *targets;
      }
      else
      {
        target_class = 0;
        real max_val = targets[ 0 ];
	  
        for( int target = 0; target < boostmachine->trainers[ trainer ]->data->n_targets; target++ )
          if( max_val < targets[ target ] )
           {
            max_val = targets[ target ];
            target_class = target;
          }
      }

      //the other ultimate test
      if( predicted_class == target_class )
      {
        boostmachine->prob_dist[ example ] *=  beta[ trainer ]  ;
      }
    }
    // normalization of the probability distribution.
    
    real sum = 0;

    for( int i = 0; i < ( data->n_examples ); i++ ){
      sum = sum + boostmachine->prob_dist[ i ];
    }

    for( int i = 0; i < ( data->n_examples ); i++ )
      boostmachine->prob_dist[ i ] = boostmachine->prob_dist[ i ] / sum; 

    //we can now pop this distribution
    //and get ready to groove with the other one.
    data->popSubset();
    
  }
  
  
}


