
#include "AdaBoostM1Machine.h"
#include "AdaBoostM1.h"
#include "math.h"

AdaBoostM1Machine::AdaBoostM1Machine( Trainer** trainers_, int n_trainers_, 
				      int n_examples_, List** trainers_measurers_ ) 
  : BoostMachine( trainers_, n_trainers_, n_examples_, trainers_measurers_ )
{

  prob_dist = ( real* )xalloc( n_examples * sizeof( real ));
  prob_inter = ( real* )xalloc( n_examples * sizeof( real ));
  
  predicted_outcome = ( int* )xalloc( n_trainers * sizeof( int ));
  sum_of_beta = ( real* )xalloc( trainer_outputs * sizeof( real ));
  
  //initialisation de la distribution de probabilite
  for( int i = 0; i < n_examples; i++ )
    prob_dist[ i ] = 1.0/n_examples;

//initialization of sum of beta parameters
  for( int i = 0; i < trainer_outputs; i++ )
    sum_of_beta[ i ] = 0;

}

AdaBoostM1Machine::~AdaBoostM1Machine()
{
  free( predicted_outcome );
}

void AdaBoostM1Machine::reset()
{

  //initialisation de la distribution de probabilite
  for( int i = 0; i < n_examples; i++ )
    prob_dist[ i ] = 1.0/n_examples;

//initialization of sum of beta parameters
  for( int i = 0; i < trainer_outputs; i++ )
    sum_of_beta[ i ] = 0;

//reset each machine that's been trained.
  for( int i = 0; i < n_trainers; i++ )
    trainers[ i ]->machine->reset();

}

/** WARNING!! this function assumes that, in a multiclass ( > 2) classification
 * problem, that the maximal output represents the predicted class by the
 * trainer. This is especially important since if this is not the case,
 * the machine will forward the given outputs as if the predicted result
 * was in fact the maximum output and therefor give false results. 
 * In a two class classification probleme, we require that the
 * algorithme has its output between -1 and 1 and that the corresponding
 * class is -1 if the output is smaller than 0 and 1 otherwise.
 */

void AdaBoostM1Machine::forward( List* inputs )
{

  real max_val;
  int predicted_class;

  for( int trainer = 0; trainer < n_trainers; trainer++ )
    {
      //let's set the ouputs of each of the trainers for the input
      //of this forward
      trainers[ trainer ]->machine->forward( inputs );
      
      //let's find the MAXIMUM of the outputs
      //what we hope to be the predicted class ;-)

      real* trainer_output = ( real* )trainers[ trainer ]->machine->outputs->ptr;
      
      if( trainers[ trainer ]->machine->n_outputs == 1)
      {
        predicted_outcome[ trainer ] = (*trainer_output > 0 )? 1 : -1;
      }
      else
      {
        predicted_class = 0;
        max_val = trainer_output[ predicted_class ];

        for( int k = 0; k < trainers[ trainer ]->machine->n_outputs; k++ )
        {
          if( max_val < trainer_output[ k ] )
          {
            max_val = trainer_output[ k ];
            predicted_class = k;
          }
        }
        predicted_outcome[ trainer ] = predicted_class;
      }
    }

  if( trainers[ 0 ]->machine->n_outputs == 1 )
  {
    real sum_of_beta_m1 = 0;
    real sum_of_beta_1 = 0;

    for( int trainer = 0; trainer < n_trainers; trainer++)
      if( predicted_outcome[ trainer ] == 1 ){
        sum_of_beta_1 += log( 1.0 / (( AdaBoostM1* )trainerBK)->beta[ trainer ]) ;
      }
      else{
        sum_of_beta_m1 += log( 1.0 / (( AdaBoostM1* )trainerBK)->beta[ trainer ]) ;
      }

    predicted_class = (sum_of_beta_m1 > sum_of_beta_1) ? -1 : 1;
    //cout << "Combined effort finds: " << predicted_class << endl;
  }
  else
    {

      for( int i = 0; i < trainers[ 0 ]->machine->n_outputs; i++)
        sum_of_beta[ i ] = 0;

      for( int classe = 0 ;  classe < trainers[ 0 ]->machine->n_outputs ; classe++ ){
        for( int trainer = 0; trainer < n_trainers; trainer++ )
          if( predicted_outcome[ trainer ] == classe ){
            //cout << "predicted_outcome[ " << trainer <<" ] " << predicted_outcome[ trainer ] << ' ' << classe << endl;
            sum_of_beta[ classe ] += log( 1.0 / (( AdaBoostM1* )trainerBK)->beta[ trainer ] );
          }
        //cout << sum_of_beta[ classe ] << endl;
      }
      
      predicted_class = 0;
      max_val = sum_of_beta[ predicted_class ];

      for( int classe = 0; classe < trainers[ 0 ]->machine->n_outputs; classe++ )
        if( max_val < sum_of_beta[ classe ] )
        {
          max_val = sum_of_beta[ classe ];
          predicted_class = classe;
        }
      //cout << "max_val: " << max_val << " predicted_class " << predicted_class << endl;
    }

  *((real*)(outputs->ptr)) = (real)predicted_class;
}









