
#include "AdaBoostM2.h"
#include "AdaBoostM2Machine.h"
#include <stdlib.h>
#include "math.h"

AdaBoostM2::AdaBoostM2( BoostMachine* machine_, DataSet* data_ )
  :Boost( machine_, data_ )
{

  beta = ( real* )xalloc( boostmachine->n_trainers*sizeof( real ));
  subset = ( int* )xalloc( data->n_examples * sizeof( int ));
  W = ( real* )xalloc( data->n_examples * sizeof( real ));
  q = ( real* )xalloc( data->n_examples * ( boostmachine->n_trainers -1 ) * sizeof( real ));
  for( int i = 0; i < boostmachine->n_trainers; i++)
    beta[ i ] = 1;

  for( int i = 0; i < data->n_examples; i++)
    W[ i ] = 0;
}

AdaBoostM2::~AdaBoostM2()
{
  free( subset ); 
  free( beta ); 
  free( W );
  free( q );
}

void AdaBoostM2::train( List* trainers_measurers )
{
  message( "Training Boosty..." );

  //initialization of the probability distribution
  //else fucks cuz reset() doesn't really know the number of examples, in the
  //event of cross-validation.

  for( int i = 0; i < ( data->n_examples * ( boostmachine->trainer_outputs - 1)); i++ )
    (( AdaBoostM2Machine* )boostmachine)->B_dist[ i ] = 1 / (real)( data->n_examples * ( boostmachine->trainer_outputs - 1));

  for( int trainer = 0; trainer < boostmachine->n_trainers; trainer++ )
  {

    real wsum = 0;
    for( int i = 0; i < data->n_examples; i++)
      W[ i ] = 0;

    //the induced probability distribution

    //set W
    for( int example = 0 ; example < data->n_examples; example++ )
    {
      int step = 0;

      for( int classe = 0; classe < boostmachine->trainer_outputs ; classe++ )
      {
        if( step < (boostmachine->trainer_outputs - 1 ) )
          W[ example ] += ( (AdaBoostM2Machine* )boostmachine)->B_dist[ (boostmachine->trainer_outputs -1)*example + step ] ;
        ++step;
        
      }
      wsum += W[ example ];
    }
    
    // set q
    for( int i = 0; i < data->n_examples; i++ )
    {
      int step = 0;
      for( int j = 0; j < boostmachine->trainer_outputs ; j++ )
      {
        if( step < (boostmachine->trainer_outputs - 1)){
          q[(boostmachine->trainer_outputs-1)*i + step ] =  
            (( AdaBoostM2Machine* )boostmachine)->B_dist[ (boostmachine->trainer_outputs-1)*i + step ] / W[ i ]; 
        }
        step++;
      }
    }
    
    //set distribution
    for( int example = 0; example < data->n_examples ; example++ )
      boostmachine->prob_dist[ example ] = W[ example ] / wsum; 

    /** get a Weaklearning rule and provide it with distribution D_t, with either sampling or weighting*/

    if( !weighting )
    {
      message(" >>> sampling <<< ");
      
      //initialization of the cumulative distribution function 
      
      boostmachine->prob_inter[ 0 ] = boostmachine->prob_dist[ 0 ];
      
      for( int i = 1 ; i < data->n_examples; i++ )
        boostmachine->prob_inter[ i ] = boostmachine->prob_inter[ i - 1 ] + boostmachine->prob_dist[ i ];

      //non-uniform sampling of the examples;
      for( int mislabelled_couple = 0; mislabelled_couple < data->n_examples; mislabelled_couple++ )
      {
        real hasard = bounded_uniform( 0, 1 );
        
        int left = 0; 
        int right = data->n_examples - 1;
        int center;

        if( hasard < boostmachine->prob_dist[ 0 ] )
        {
          subset[ mislabelled_couple ] = 0;
          continue;
        }
        
        while( ( left + 1 ) != right )
        {
          center = ( right + left ) / 2;
          if( hasard < boostmachine->prob_inter[ center ])
            right = center;
          else
            left = center;
        }
      
        subset[ mislabelled_couple ] = right;
      }
      
    }
    else
    {

      message(" >>> weighting <<< ");
      
      GMTrainer* tmp_trainer = ( GMTrainer* ) boostmachine->trainers[ trainer ];
      WeightedMseCriterion* tmp_criterion = ( WeightedMseCriterion* ) ( tmp_trainer->criterion );

      //lets find the max of the weights 
      real max = boostmachine->prob_dist[ 0 ];
      
      for( int i = 1; i < data->n_examples; i++)
        if( max < boostmachine->prob_dist[ i ] )
          max = boostmachine->prob_dist[ i ];

      for( int i = 0; i < data->n_examples; i++ )
        tmp_criterion->weights[ i ] = boostmachine->prob_dist[ i ] / max ;
      
      for( int i = 0; i < data->n_examples; i++ )
        subset[ i ] = i;
    }

    /** train this rule with the given distribution 
     * with the associated measurer. 
     */
    
    //push the distribution on the data set.
    data->pushSubset( subset, data->n_examples );

    boostmachine->trainers[ trainer ]->machine->reset();
    boostmachine->trainers[ trainer ]->train( boostmachine->trainers_measurers[ trainer ]);

    //we can now pop the sampled distribution
    data->popSubset();

    /** calculate the pseudo-loss of this Weaklearner */

    
    //we must reselect the examples
    for( int example = 0; example < data->n_examples; example++ )
      subset[ example ] = example;

    //push this distribution
    data->pushSubset( subset, data->n_examples );

    real pseudo_loss = 0;

    for( int example = 0; example < data->n_examples; example++ )
    {
      
      boostmachine->trainers[ trainer ]->data->setExample( example );
      boostmachine->trainers[ trainer ]->machine->forward( boostmachine->trainers[ trainer ]->data->inputs );
      
      real* trainer_output = ( real* ) boostmachine->trainers[ trainer ]->machine->outputs->ptr;
      
      //lets find this examples' class

      /** WARNING! We assume here that the examples' target are in 
       * such a way that that the maximal value of the target vector
       * represents the examples' class
       */
      
      real* targets = ( real* ) boostmachine->trainers[ trainer ]->data->targets;
      
      int target_class = 0;
      real max_val = targets[ target_class ];

      for( int target = 0; target < data->n_targets; target++ )
        if( max_val < targets[ target ] )
        {
          max_val = targets[ target ];
          target_class = target;
	      }

      real ysum = 0;
      int step = 0;

      for( int classe = 0; classe < boostmachine->trainer_outputs; classe++ )
      {
        if( classe != target_class )
        {
          ysum += ( q[ (boostmachine->trainer_outputs - 1)*example + step ]*trainer_output[ classe ] );
          ++step;
        }
      }
      pseudo_loss += boostmachine->prob_dist[ example ] * ( 1 - trainer_output[ target_class ] + ysum );
    }
    
    pseudo_loss = pseudo_loss / 2;

    message("error: %f\n",pseudo_loss);

    //if the error is more than one half, set T = t - 1 and abort loop
    if( pseudo_loss > 0.5 )
    {
      message( "Weaklearner too weak. Aborting Boosting." );
      trainer = boostmachine->n_trainers - 1;
      data->popSubset();
      continue;
    }
    
    /** set beta */
    if( pseudo_loss == 0 )
      beta[ trainer ] = 1.0/( real ) RAND_MAX;
    else
      beta[ trainer ] = pseudo_loss / ( 1.0 - pseudo_loss );
      
    /** update the distribution */
      
    for( int example = 0; example < data->n_examples; example++ )
    {
      boostmachine->trainers[ trainer ]->data->setExample( example );
      boostmachine->trainers[ trainer ]->machine->forward( boostmachine->trainers[ trainer ]->data->inputs );
      
      real* trainer_output = ( real* )boostmachine->trainers[ trainer ]->machine->outputs->ptr;
      
      //the examples' class. Same thing as before
      real* targets = ( real* ) boostmachine->trainers[ trainer ]->data->targets;
      
      int target_class = 0;
      real max_val = targets[ target_class ];
	  
      for( int target = 0; target < data->n_targets; target++ )
        if( max_val < targets[ target ] )
	      {
          max_val = targets[ target ];
          target_class = target;
	      }
      
      int step = 0;
      for( int classe = 0; classe < boostmachine->trainer_outputs; classe++ ){
        if( classe != target_class )
	      {
          (( AdaBoostM2Machine* )boostmachine)->B_dist[ (boostmachine->trainer_outputs-1)*example + step ] =  
            (( AdaBoostM2Machine* )boostmachine)->B_dist[ (boostmachine->trainer_outputs-1)*example + step ] * 
            pow( beta[ trainer ], 0.5 * ( 1 + trainer_output[ target_class ] - trainer_output[ classe ]));
          
          ++step;
          
        }
      }
    }
    
    //we can now push our distribution
    data->popSubset();

  }

}


