
#include "AdaBoostM2Machine.h"
#include "AdaBoostM2.h"
#include "math.h"

AdaBoostM2Machine::AdaBoostM2Machine( Trainer** trainers_, int n_trainers_, 
				      int n_examples_, List** measurers_) 
  : BoostMachine( trainers_, n_trainers_, n_examples_ , measurers_ )
{

  B_dist = ( real* )xalloc( ( n_examples * ( trainer_outputs - 1 )) * sizeof( real ));
  B_inter = ( real* )xalloc( ( n_examples * ( trainer_outputs - 1 )) * sizeof( real ));

  prob_dist = ( real* )xalloc( n_examples * sizeof( real ));
  prob_inter = ( real* ) xalloc( n_examples * sizeof( real ));

  sum_of_beta = ( real* )xalloc( trainer_outputs * sizeof( real ));

  //initialization of the probability distribution
  for( int i = 0; i < ( n_examples * ( trainer_outputs - 1)); i++ )
  {
    B_dist[ i ] = 1.0 / ( n_examples * ( trainer_outputs - 1));
  }
  
  //initialization of sum of beta parameters
  for( int i = 0; i < trainer_outputs; i++ )
    sum_of_beta[ i ] = 0;

  //intialize the probability distribution.
  for( int i = 0 ; i < n_examples ; i++ )
    prob_dist[ i ] = 1.0/ n_examples;
  
}

AdaBoostM2Machine::~AdaBoostM2Machine()
{
  free( B_dist );
  free( B_inter );
}

void AdaBoostM2Machine::reset()
{

  //initialization of the probability distribution
  for( int i = 0; i < ( n_examples * ( trainer_outputs - 1)); i++ )
  {
    B_dist[ i ] = 1.0 / ( n_examples * ( trainer_outputs - 1));
  }

//reset each machine that's been trained.
  for( int i = 0; i < n_trainers; i++ )
    trainers[ i ]->machine->reset();

  //intialize the probability distribution.
  for( int i = 0 ; i < n_examples ; i++ )
    prob_dist[ i ] = 1.0/ n_examples;


}



void AdaBoostM2Machine::forward( List* inputs )
{

  //initialization of sum_of_betas
  for( int i = 0; i < trainer_outputs; i++)
    sum_of_beta[ i ] = 0;

  for( int trainer = 0; trainer < n_trainers; trainer++ )
    {
      //set the trainers' outputs
      trainers[ trainer ]->machine->forward( inputs );
      
      real* trainer_output = ( real* )trainers[ trainer ]->machine->outputs->ptr;
      
      for( int classe = 0; classe < trainer_outputs; classe++ )
      {
        sum_of_beta[ classe ] += log( 1.0/ (( AdaBoostM2* )trainerBK)->beta[ trainer ])
          * trainer_output[ classe ];
      }
      
    }
  
  //find argmax.
  
  int predicted_class = 0;
  real max_val = sum_of_beta[ predicted_class ];
  
  for( int classe = 0; classe < trainer_outputs ; classe++ )
    if( max_val < sum_of_beta[ classe ] )
    {
      max_val = sum_of_beta[ classe ];
      predicted_class = classe;
    }
  

  
  *((real*)(outputs->ptr)) = (real)predicted_class;
    
}


