#include "Bagging.h"
#include "random.h"

Bagging::Bagging(BaggingMachine* b_machine_, DataSet* data_) : Trainer(b_machine_,data_)
{
  b_machine = b_machine_;

  int n_examples = data->n_examples;
  n_trainers = b_machine->n_trainers;
  is_selected_examples = (int *)xalloc(n_examples*sizeof(int));
  n_unselected_examples = (int *)xalloc(n_trainers*sizeof(int));
  unselected_examples = (int **)xalloc(n_trainers*sizeof(int*));
  selected_examples = (int **)xalloc(n_trainers*sizeof(int*));
  for (int i=0;i<n_trainers;i++) {
    unselected_examples[i] = (int *)xalloc(n_examples*sizeof(int));
    selected_examples[i] = (int *)xalloc(n_examples*sizeof(int));
  } 

}

void Bagging::train(List* measurers)
{
  message("Bagging: training");

  int n = data->n_examples;
  for (int i=0;i<n_trainers;i++) {
    // initialization
    for (int j=0;j<n;j++) {
      is_selected_examples[j]=0;
    }

    // select a bootstrap
    for (int j=0;j<n;j++) {
      selected_examples[i][j] = (int)floor(bounded_uniform(0,n));
      is_selected_examples[selected_examples[i][j]] = 1;
    }
    b_machine->trainers[i]->data->pushSubset(selected_examples[i],n);

    // keep in mind examples not used by trainers[i]
    int k=0;
    for (int j=0;j<n;j++) {
      if (!is_selected_examples[j])
        unselected_examples[i][k++] = j;
    }
    n_unselected_examples[i] = k;
    

    // train the trainer
    b_machine->trainers[i]->train(b_machine->trainers_measurers ? b_machine->trainers_measurers[i] : NULL);

    // put back the selected_examples
    b_machine->trainers[i]->data->popSubset();

    // if measurers is given, call the test method by fooling it
    // with the number of trainers
    if (measurers) {
      b_machine->n_trainers = i+1;
      test(measurers);
      b_machine->n_trainers = n_trainers;
    }
  }
}

Bagging::~Bagging()
{
  free(is_selected_examples);
  free(n_unselected_examples);
  for (int i=0;i<n_trainers;i++) {
    free(unselected_examples[i]);
    free(selected_examples[i]);
  }
  free(selected_examples);
  free(unselected_examples);
}
