#ifndef BAGGING_INC
#define BAGGING_INC

#include "Trainer.h"
#include "Measurer.h"
#include "DataSet.h"
#include "BaggingMachine.h"

/** This class represents a #Trainer# that implements the well-known
    Bagging algorithm (Breiman, 1996). A "bagger" contains a series
    of trainers, each trained on a bootstrap of the original dataset.
    The output of the bagging is then the average of the output of
    each trainer.

    It is implemented using a #BaggingMachine# that performs the combination.

    @author Samy Bengio (bengio@idiap.ch)
*/
class Bagging : public Trainer
{
  public:

    /// This machine performs the combination. It contains many trainers.
    BaggingMachine* b_machine;

    /// The number of trainers in the bagging.
    int n_trainers;

    /// for each trainer, keep the indices of examples not used during training
    int** unselected_examples;
    /// for each trainer, keep the indices of examples used during training
    int** selected_examples;
    /// for each trainer, keep the number of examples not used during training
    int* n_unselected_examples;
    /// for each trainer, keep the number of examples used during training
    int* is_selected_examples;
 
    ///
    Bagging(BaggingMachine *b_machine_, DataSet* data_);

    virtual void train(List *measurers);

    virtual ~Bagging();
};

#endif
