#ifndef BAGGING_MACHINE_INC
#define BAGGING_MACHINE_INC

#include "Machine.h"
#include "Trainer.h"
#include "DataSet.h"

/** This class implements the bagging algorithm. It must be used in
    conjunction with the #Bagging# trainer. It contains a series of
    Trainers, and its forward method simply performs the average
    of the output of each trainer on the same input.

    @author Samy Bengio (bengio@idiap.ch)
*/
class BaggingMachine : public Machine
{
  public:

    /// the trainers that are "bagged"
    Trainer **trainers;
    /// the corresponding measurers
    List **trainers_measurers;
    /// the number of trainers in the bagging
    int n_trainers;

    ///
    BaggingMachine(Trainer **trainer_, int n_trainers_, List **trainers_measurers);

    virtual void forward(List *inputs);

    virtual void init();
    virtual void reset();
    virtual void loadFILE(FILE *file);
    virtual void saveFILE(FILE *file);

    virtual ~BaggingMachine();
};

#endif
