#ifndef BAYES_CLASSIFIER_MACHINE_INC
#define BAYES_CLASSIFIER_MACHINE_INC

#include "Machine.h"
#include "Trainer.h"
#include "ClassFormat.h"

/** BayesClassifierMachine is the machine used by the #BayesClassifier#
    trainer to perform a Bayes Classification using different distributions
    The output correspond to the class that is the most probable
    (using prior AND posterior information).

    @author Bison Ravi (francois.belisle@idiap.ch)
    @author Samy Bengio (bengio@idiap.ch)
 */
class BayesClassifierMachine : public Machine
{
  public:
    
    /// the number of classes corresponds to the number of #Trainer#
    int n_trainers; 

    /// the actual trainers.
    Trainer** trainers; 

    /** the log_prior probabilities of each class. default: log_priors are
        taken as the log of the proportions in the training set.
    */
    real* log_priors; 
    
    /// contains the log posterior probability plus the log prior of the class
    real* log_posteriors;
    List* list_log_posteriors;

    /// used to know if log_priors where given or allocated
    bool allocated_log_priors;

    /// the format of the data
    ClassFormat* class_format;

    /// the measurers for each individual trainer
    List** trainers_measurers; 

    BayesClassifierMachine( Trainer**, int n_trainers_, List** trainers_measurers_ , ClassFormat* class_format_, real* log_priors_=NULL);
    virtual ~BayesClassifierMachine();

    /** definition of virtual functions of #Machine# */
    virtual void forward( List* inputs );
    virtual void init();
    virtual void reset();
    virtual void loadFILE( FILE* );
    virtual void saveFILE( FILE* );

};

#endif
