#include "BinFileDataSet.h"

BinFileDataSet::BinFileDataSet(char *data_file,char *target_file, int n_cols_,int n_target_,int window_size_, int max_load)
{
  window_size = window_size_;
  n_files = 1;
  n_inputs = n_cols_*window_size;
  n_targets = n_target_;
  bin = (IOBin**)xalloc(sizeof(IOBin*)*n_files);
  bin[0] = new IOBin(data_file,target_file,n_cols_,max_load);
  all_inputs = NULL;
  all_targets = NULL;
  prepareData();
}

BinFileDataSet::BinFileDataSet(char **data_files,char** target_files, int n_files_,int n_cols_,int n_target_, int window_size_, int max_load)
{
  window_size = window_size_;
  n_files = n_files_;
  n_inputs = n_cols_*window_size;
  n_targets = n_target_;
  bin = (IOBin**)xalloc(sizeof(IOBin*)*n_files);
  for (int i=0;i<n_files;i++) {
		bin[i] = new IOBin(data_files[i],target_files[i],n_cols_,max_load);
  }
  all_inputs = NULL;
  all_targets = NULL;
  prepareData();
}

void BinFileDataSet::prepareData()
{
  // first count the number of examples
  n_real_examples = 0;
  for (int i=0;i<n_files;i++) {
    n_real_examples += bin[i]->n_lines - window_size + 1;
  }

  // then create all_inputs and all_targets
  all_inputs = (real**)xrealloc(all_inputs,sizeof(real*)*n_real_examples);
	if(!all_targets)
		all_targets = (real**)xrealloc(all_targets,sizeof(real*)*n_real_examples);
  int k=0;
  for (int i=0;i<n_files;i++) {
    for (int j=0;j<bin[i]->n_lines - window_size + 1;j++){
			all_inputs[k] = &bin[i]->data[j*n_inputs/window_size];
			if(!all_targets[k]){
				all_targets[k] = (real*)xcalloc(n_targets,sizeof(real));
				int index = j+(int)(window_size/2);
				all_targets[k++][bin[i]->target[index]] = 1.0;
			}
		}
  }
	n_examples = n_real_examples = k;
  //all_targets = all_inputs;
}

void BinFileDataSet::init()
{
  DataSet::init();
  normalize();

  addToList(&inputs, n_inputs, NULL);
}

void BinFileDataSet::normalize()
{
  // we first need to fool the StdDataSet::normalize by modifying
  // the number of window_size
  if (window_size == 1) {
    StdDataSet::normalize();
  } else {
    int old_n_real_examples = n_real_examples;
    int old_n_inputs = n_inputs;
    n_inputs = n_inputs/window_size;
    //n_targets = n_inputs;
    n_real_examples = 0;
    for (int i=0;i<n_files;i++) {
      n_real_examples += bin[i]->n_lines;
    }
    real** old_all_inputs = all_inputs;
    all_inputs = (real**)xalloc(sizeof(real*)*n_real_examples);
    int k=0;
    for (int i=0;i<n_files;i++) {
      for (int j=0;j<bin[i]->n_lines;j++,k++)
        all_inputs[k] = &bin[i]->data[j*n_inputs];
    }
    //all_targets = all_inputs;
    StdDataSet::normalize();
    free(all_inputs);
    all_inputs = old_all_inputs;
    //all_targets = all_inputs;
    n_real_examples = old_n_real_examples;
    n_inputs = old_n_inputs;
    //n_targets = n_inputs;
  }
}

void BinFileDataSet::normalizeUsingDataSet(StdDataSet *data_norm)
{
  // we first need to fool the StdDataSet::normalize by modifying
  // the number of window_size
  if (window_size == 1) {
    StdDataSet::normalizeUsingDataSet(data_norm);
  } else if (data_norm->norm_inputs) {
    int old_n_real_examples = n_real_examples;
    int old_n_inputs = n_inputs;
    n_inputs = n_inputs/window_size;
    //n_targets = n_inputs;
    n_real_examples = 0;
    for (int i=0;i<n_files;i++) {
      n_real_examples += bin[i]->n_lines;
    }
    real** old_all_inputs = all_inputs;
    all_inputs = (real**)xalloc(sizeof(real*)*n_real_examples);
    int k=0;
    for (int i=0;i<n_files;i++) {
      for (int j=0;j<bin[i]->n_lines;j++,k++)
      all_inputs[k] = &bin[i]->data[j*n_inputs];
    }
    //all_targets = all_inputs;

    for(int i = 0; i < n_real_examples; i++) {
      for(int d = 0; d < n_inputs; d++)
        all_inputs[i][d] = (all_inputs[i][d]-data_norm->mean_i[d])/data_norm->stdv_i[d];
    }

    free(all_inputs);
    all_inputs = old_all_inputs;
    //all_targets = all_inputs;
    n_real_examples = old_n_real_examples;
    n_inputs = old_n_inputs;
    //n_targets = n_inputs;
  }
}


void BinFileDataSet::applyMask(bool* mask){
	if(mask){
		for(int i=0;i<n_files;i++){
			IOBin* h = bin[i];
			real* vect = h->data;
			int k=0;
			for(int j=0;j<h->n_lines;j++){
				for(int l=0;l<h->n_cols;l++)
					if(!mask[l]){
						vect[k] = vect[l+h->n_cols*j];
						k++;
					}
			}
		}
		n_inputs = 0;
		for(int l=0;l<bin[0]->n_cols;l++)
			if(!mask[l]){
				n_inputs++;
			}
		n_inputs = n_inputs * window_size;
	}
	prepareData();
}

void BinFileDataSet::loadFILE(FILE *file)
{
  if(norm_inputs) {
    int old_n_real_examples = n_real_examples;
    int old_n_inputs = n_inputs;
    n_inputs = n_inputs/window_size;
    //n_targets = n_inputs;
    n_real_examples = 0;
    for (int i=0;i<n_files;i++) {
      n_real_examples += bin[i]->n_lines;
    }
    real** old_all_inputs = all_inputs;
    all_inputs = (real**)xalloc(sizeof(real*)*n_real_examples);
    int k=0;
    for (int i=0;i<n_files;i++) {
      for (int j=0;j<bin[i]->n_lines;j++,k++)
      all_inputs[k] = &bin[i]->data[j*n_inputs];
    }
    //all_targets = all_inputs;

    for(int i = 0; i < n_real_examples; i++) {
      for(int d = 0; d < n_inputs; d++)
        all_inputs[i][d] = all_inputs[i][d]*stdv_i[d]+mean_i[d];
    }

    xfread(mean_i, sizeof(real), n_inputs, file);
    xfread(stdv_i, sizeof(real), n_inputs, file);
  
    for(int i = 0; i < n_real_examples; i++) {
      for(int d = 0; d < n_inputs; d++)
        all_inputs[i][d] = (all_inputs[i][d]-mean_i[d])/stdv_i[d];
    }

    free(all_inputs);
    all_inputs = old_all_inputs;
    //all_targets = all_inputs;
    n_real_examples = old_n_real_examples;
    n_inputs = old_n_inputs;
    //n_targets = n_inputs;
  }
}

void BinFileDataSet::saveFILE(FILE *file)
{
  if(norm_inputs) {
    xfwrite(mean_i, sizeof(real), n_inputs/window_size, file);
    xfwrite(stdv_i, sizeof(real), n_inputs/window_size, file);
  }
}


BinFileDataSet::~BinFileDataSet()
{
  for (int i=0;i<n_files;i++)
    delete bin[i];
  free(bin);
  free(all_inputs);
	for(int i=0;i<n_real_examples;i++)
		free(all_targets[i]);
	free(all_targets);
}
