#ifndef BIN_FILE_DATA_SET_INC
#define BIN_FILE_DATA_SET_INC

#include "StdDataSet.h"
#include "IOBin.h"
#include "Distribution.h"

/** Create a #DataSet# from a disk file in bin format.

    @author Samy Bengio (bengio@idiap.ch)
 */
class BinFileDataSet : public StdDataSet
{
  public:
    int n_files;
    IOBin** bin;
    int window_size;

    //-----

    /** Load #file# in memory.
        \begin{itemize}
          \item #window_size# gives the number of frames per input.
          \item if #max_load# > 0, it loads only the first #max_load# examples.
        \end{itemize}
    */
    BinFileDataSet(char *data_file,char *target_file, int n_cols_,int n_target_,int window_size_ = 1, int max_load=-1);
    BinFileDataSet(char **data_files,char** target_file, int n_files_,int n_cols_,int n_target_, int window_size_ = 1, int max_load=-1);

    virtual void prepareData();
		virtual void applyMask(bool* mask);
    virtual void normalizeUsingDataSet(StdDataSet *data_norm);
    virtual void normalize();
    virtual void init();

    /// Load the normalization parameters
    virtual void loadFILE(FILE *file);
    /// Save the normalization parameters
    virtual void saveFILE(FILE *file);

    virtual ~BinFileDataSet();
};

#endif

