#ifndef BOOST_MACHINE_INC
#define BOOST_MACHINE_INC

#include "Trainer.h"

/** 
 * the Boost Machine. The central point of this class
 * is the method forward which is responsible of 
 * forwarding the information gathered by the 
 * Boost Trainer and hence each of the weaklearners ( or
 * in english: setting the ouputs... )
 * This method will be overloaded as each Boosting
 * algorithm has its own way of interpretating the outputs
 * of each weaklearner.
 *
 * @author Bison Ravi (francois.belisle@idiap.ch)
 */
class BoostMachine : public Machine
{
  public:
  
    Trainer* trainerBK; //the Big Kahoona: trainer of trainers. 
    Trainer** trainers; //all of the trainers
        
    List** trainers_measurers; //the measurers for each individual trainer.

    /** the number of WeakLearners that will be called by 
     * the boosting algorithm */
    int n_trainers; 
    int n_examples; //all the examples in the DataMachine

    int trainer_outputs; //the number of outputs of each trainer.
  
    real* prob_dist; //the probability distribution of the examples
    real* prob_inter; //the probability intervals (to be used for non-uniforme sampling)

    real* sum_of_beta; //for computational ends

    BoostMachine( Trainer**, int n_trainers_, int n_examples_, List** trainers_measurers );
    virtual ~BoostMachine();

    /** this sets the trainer of trainers. Its main
     * use is to have access to all of the fields
     * in the Trainer, such as the "error" and 
     * "pseudo-loss" calculated by the trainer.
     * it must be set before fowarding can be
     * done.
     */
    void setTrainer( Trainer* ); 
  
    /** definition of the virtual functions of #Machine# */
    virtual void init();
    virtual void reset();
    virtual void loadFILE( FILE* );
    virtual void saveFILE( FILE* );
  
};

#endif


