#include "Boosting.h"
#include "random.h"

void randw(int *selected_examples, real *ex_weights, int n_examples)
{
  real *repartition = (real *)xalloc(sizeof(real)*(n_examples+1));
  repartition[0] = 0;
  for(int i = 0; i < n_examples; i++)
    repartition[i+1] = repartition[i]+ex_weights[i];

  for(int i = 0; i < n_examples; i++)
  {
    real z = uniform();
    int gauche = 0;
    int droite = n_examples;
    while(gauche+1 != droite)
    {
      int centre = (gauche+droite)/2;
      if(repartition[centre] < z)
        gauche = centre;
      else
        droite = centre;
    }
    selected_examples[i] = gauche;
//    printf("%g < %g < %g\n", repartition[gauche], z, repartition[gauche+1]);
  }
  free(repartition);
}

Boosting::Boosting(BoostingMachine* b_machine_, DataSet* data_, BoostingMeasurer *measurer_) : Trainer(b_machine_, data_)
{
  b_machine = b_machine_;
  measurer = measurer_;

  n_trainers = b_machine->n_trainers;
  weights = (real *)xalloc(n_trainers*sizeof(real));
  b_machine->weights = weights;
  for(int i = 0; i < n_trainers; i++)
    weights[i] = 0;
}

void Boosting::train(List* measurers)
{
  int n_examples = data->n_examples;
  int *selected_examples = (int *)xalloc(n_examples*sizeof(int));
  real *ex_weights = (real *)xalloc(n_examples*sizeof(real));
  for(int t = 0; t < n_examples; t++)
    ex_weights[t] = 1./((real)n_examples);

  measurer->setData(data);
  measurer->setWeights(ex_weights);

  message("Boosting: training...");

  List *the_boost_mes = NULL;
  addToList(&the_boost_mes, 1, measurer);
  for(int i = 0; i < n_trainers; i++)
  {
    randw(selected_examples, ex_weights, n_examples);
    data->pushSubset(selected_examples, n_examples);
    b_machine->trainers[i]->train(b_machine->trainers_measurers[i]);
    data->popSubset();

    // Find beta and all missclass
    measurer->setInputs(b_machine->trainers[i]->machine->outputs);
    b_machine->trainers[i]->test(the_boost_mes);

    // Compute new weights
    int *ptr_status = measurer->status;
    real *ptr_ex_weights = ex_weights;
    real beta = measurer->beta;
    for(int t = 0; t < n_examples; t++)
    {
      if(*ptr_status++ > 0)
        *ptr_ex_weights *= beta;
      ptr_ex_weights++;
    }

    // Normalize
    ptr_ex_weights = ex_weights;
    real z = 0;
    for(int t = 0; t < n_examples; t++)
      z += *ptr_ex_weights++;

    ptr_ex_weights = ex_weights;
    for(int t = 0; t < n_examples; t++)
      *ptr_ex_weights++ /= z;

    // Warning: no precautions, no normalization
    weights[i] = -log(beta);

    b_machine->n_trainers = i+1;
    test(measurers);
  }

  freeList(&the_boost_mes);
  free(selected_examples);
}

Boosting::~Boosting()
{
}
