#include "ConfidenceBagMeasurer.h"

ConfidenceBagMeasurer::ConfidenceBagMeasurer(BaggingMachine* b_machine_, DataSet *data_, FILE *file_, real level_) : Measurer(data_,file_)
{
  measure = xalloc(sizeof(real)*2);

  data = data_;
  level = level_;
  b_machine = b_machine_;

  reset();
}

ConfidenceBagMeasurer::ConfidenceBagMeasurer(BaggingMachine* b_machine_, DataSet *data_, const char *filename, real level_) : Measurer(data_,filename)
{
  measure = xalloc(sizeof(real)*2);

  data = data_;
  level = level_;
  b_machine = b_machine_;

  reset();
}

extern "C" int compar_int_real(const void *p1, const void *p2)
{
  Int_real v1 = *((Int_real*)p1);
  Int_real v2 = *((Int_real*)p2);
  if (v1.the_real > v2.the_real)
    return 1;
  if (v1.the_real < v2.the_real)
    return -1;
  return 0;
}

void ConfidenceBagMeasurer::measureEx()
{

  int n_trainers = b_machine->n_trainers;

  for (int i=0;i<n_trainers;i++) {
    real *out_from = (real*)b_machine->trainers[i]->machine->outputs->ptr;
    real *out_bag = (real*)b_machine->outputs->ptr;
    real dist = 0;
    for (int j=0;j<b_machine->n_outputs;j++) {
      dist += fabs(*out_from++ - *out_bag++);
    }
    to_sort[i].the_int = i;
    to_sort[i].the_real = dist;
  }
  qsort(to_sort,n_trainers,sizeof(Int_real),compar_int_real);

  // print the output
  real *out_bag = (real*)b_machine->outputs->ptr;
  fprintf(file,"out ");
  for (int i=0;i<b_machine->n_outputs;i++)
    fprintf(file,"%g ",*out_bag++);

  // print the target
  real *des = (real*)data->targets;
  fprintf(file,"target ");
  for (int i=0;i<b_machine->n_outputs;i++)
    fprintf(file,"%g ",*des++);

  // find the interval
  int bound = (int)(level*b_machine->n_trainers);
  real *low_b = NULL;
  real *up_b = NULL;
  real *a_b = (real*)b_machine->trainers[to_sort[bound].the_int]->machine->outputs->ptr;
  while (!up_b || !low_b) {
    // is a_b a lower bound or an upper bound?
    out_bag = (real*)b_machine->outputs->ptr;
    real dist = 0;
    for (int j=0;j<b_machine->n_outputs;j++) {
      dist += *a_b++ - *out_bag++;
    }
    if (dist > 0 && !up_b)
      up_b = (real*)b_machine->trainers[to_sort[bound].the_int]->machine->outputs->ptr;
    else if (dist <=0 && !low_b)
      low_b = (real*)b_machine->trainers[to_sort[bound].the_int]->machine->outputs->ptr;
    bound--;
    a_b = (real*)b_machine->trainers[to_sort[bound].the_int]->machine->outputs->ptr;
  }

  //print the interval
  fprintf(file,"lower ");
  real *low_bound = low_b;
  for (int i=0;i<b_machine->n_outputs;i++)
    fprintf(file,"%g ",*low_bound++);
  fprintf(file,"upper ");
  real *up_bound = up_b;
  for (int i=0;i<b_machine->n_outputs;i++)
    fprintf(file,"%g ",*up_bound++);

  // is the target in the interval?
  bool target_in = true;
  // it has to be more or equal than the lower bound
  des = (real*)data->targets;
  low_bound = low_b;
  for (int i=0;i<b_machine->n_outputs && target_in;i++) {
    if (*des++ < *low_bound++)
      target_in = false;
  }
  // it has to be less or equal than the upper bound
  des = (real*)data->targets;
  up_bound = up_b;
  for (int i=0;i<b_machine->n_outputs && target_in;i++) {
    if (*des++ > *up_bound++)
      target_in = false;
  }
  fprintf(file,"target_in_interval %d\n",target_in);
}

void ConfidenceBagMeasurer::measureIter()
{
}

void ConfidenceBagMeasurer::init()
{
  to_sort = (Int_real*)xalloc(b_machine->n_trainers*sizeof(Int_real));

  reset();
}

void ConfidenceBagMeasurer::reset()
{
}

ConfidenceBagMeasurer::~ConfidenceBagMeasurer()
{
  free(measure);
  free(to_sort);
}

