#include "CovParzenMachine.h"
#include "Mat_operations.h"
#include "mx_lu_factor.h"

CovParzenMachine::CovParzenMachine(DataSet* data_,real **cov_)
{
  data = data_;
  mat_cov = new Mat(cov_,data->n_inputs,data->n_inputs);
  mat_inv_cov = new Mat(data->n_inputs,data->n_inputs);
  setCov(cov_);
  vec_tmp = new Vec(data->n_inputs);
  vec_tmp2 = new Vec(data->n_inputs);
  n_outputs = data->n_targets;
  n_inputs = data->n_inputs;
  addToList(&outputs, n_outputs, (real *)xalloc(sizeof(real)*n_outputs));
  n_real_examples = 0;
  real_examples = NULL;
}

void CovParzenMachine::setCov(real **cov_)
{
  cov = cov_;
  mat_cov->ptr = cov;
  mxInverse(mat_cov,mat_inv_cov);
}

void CovParzenMachine::init()
{
  // do it your self in the main
  //reset();
}

void CovParzenMachine::reset()
{
  data->setExample(0);
  n_real_examples = data->n_examples;
  real_examples = (int*)xrealloc(real_examples,n_real_examples*sizeof(int));
  for (int i=0;i<data->n_examples;i++) {
    real_examples[i] = data->selected_examples[i];
  }
}

void CovParzenMachine::forward(List* inputs)
{
  // keep current example in order to restore at the end
  int current_example = data->current_example;

  real* out = (real*)outputs->ptr;
  denominator = 0.;
  for (int j=0;j<n_outputs;j++)
    *out++ = 0;
  int *i_ptr = real_examples;
  for (int i=0;i<n_real_examples;i++) {
    data->setRealExample(*i_ptr++);
    real* in = (real*)inputs->ptr;
    real* in_i = (real*)data->inputs->ptr;
    real* tmp = vec_tmp->ptr;
    for (int j=0;j<n_inputs;j++) {
      *tmp++ = *in++ - *in_i++;
    }
    mxVecMulMat(vec_tmp,mat_inv_cov,vec_tmp2);
    tmp = vec_tmp->ptr;
    real* tmp2 = vec_tmp2->ptr;
    real e = 0;
    for (int j=0;j<n_inputs;j++) {
      e += *tmp++ * *tmp2++;
    }
    e *= 0.5;
    denominator += e;
    out = (real*)outputs->ptr;
    real* targ_i = (real*)data->targets;
    for (int j=0;j<n_outputs;j++) {
      *out++ += *targ_i++ * e;
    }
  }

  out = (real*)outputs->ptr;
  for (int j=0;j<n_outputs;j++)
    *out++ /= denominator;

  // restore current_example
  data->setRealExample(current_example);
}

CovParzenMachine::~CovParzenMachine()
{
  delete mat_cov;
  delete mat_inv_cov;
  delete vec_tmp;
  delete vec_tmp2;
  freeList(&outputs,true);
  free(real_examples);
}
