#ifndef COV_PARZEN_MACHINE_INC
#define COV_PARZEN_MACHINE_INC

#include "Machine.h"
#include "DataSet.h"
#include "Mat.h"

/** This machine implements the Parzen Window estimator with covariance matrix.
    Given a dataset (in the constructor), the #forward# method returns
    for a given input the average of the outputs of the training set
    weighted by a Gaussian kernel distance:

    $ y(x) = \frac{\sum_i t_i \exp(- 0.5 ||x - x_i||^T Cov ||x - x_i||)}{\sum_i \exp(- 0.5 ||x - x_i||^T Cov ||x - x_i||)}$

    The only parameter #Cov# is a matrix given in the constructor.

    @author Samy Bengio (bengio@idiap.ch)
*/
class CovParzenMachine : public Machine
{
  public:

    /// the covariance used
    real **cov;

    /// pointers used for matrices operations
    Mat* mat_cov;
    Mat* mat_inv_cov;
    Vec* vec_tmp;
    Vec* vec_tmp2;

    /// The dataset that contains the training set
    DataSet* data;

    /// the indices of the training examples
    int *real_examples;
    int n_real_examples;

    /// keep the denominator
    real denominator;

    ///
    CovParzenMachine(DataSet* data_,real **cov_);

    virtual void forward(List *inputs);
    virtual void init();
    virtual void reset();

    /// change the value of cov
    virtual void setCov(real **cov_);

    virtual ~CovParzenMachine();
};

#endif
