#ifndef DICTIONARY_INC
#define DICTIONARY_INC

#include "general.h"
#include "Object.h"

/** This class contains the dictionary of accepted words for 
    a speech recognition experiment, such as the one used by SpeechHMM.
    It contains a list of words, each word being a list of phonemes.

    @author Samy Bengio (bengio@idiap.ch)
*/
class Dictionary : public Object
{
  public:
    /// the vector of words, which are represented by vectors of phonemes
    int** words;
    /// the length of each word (number of phonemes)
    int* word_length;
    /// the number of words in the dictionary
    int n_words;
    /// for each word, the literal transcription
    char** words_string;
    /// the index of the "silence" word
    int silence_word;

    /// the phonemes
    char** phonemes;
    /// the number of different phonemes
    int n_phonemes;

    ///
    Dictionary();
    ///
    Dictionary(char* filename, char** phonemes, int n_phonemes);

    /// this methods return the index of the given word, if found
    virtual int findWord(char* word);

    /// this methods return the index of the given phoneme, if found
    virtual int findPhoneme(char* phoneme);

    virtual ~Dictionary();
};

/** This class contains the grammar of accepted sentences for a
    speech recognition experiment such as the one using SpeechHMM
    A grammar is a transition table where each node is a word.

    @author Samy Bengio (bengio@idiap.ch)
*/
class Grammar : public Object
{
  public:
    /** the number of words in the grammar (different than the number
        of words in the dictionary, because the same word can appear twice
        in the grammar)
    */
    int n_words;
    /// the index of the words (in the dictionary object)
    int* words;
    /** this vector is used in SpeechHMM to keep the state index in the
        decoding state matrix, corresponding to the given word
    */
    int* start;
    /** the transition matrix. each true transition (i,j) means the
        word whose index is i can be followed by the word whose index is j
    */
    bool** transitions;

    ///
    Grammar(int n_words_);
    virtual ~Grammar();
};

#endif
