#ifndef DISTR_MACHINE_INC
#define DISTR_MACHINE_INC

#include "Distribution.h"

/** This class can be used to implement a conditional distribution P(y|x;theta).
    It is represented as a machine followed by a distribution. The
    machine, given some inputs "x", computes the parameters "theta"
    of the distribution, then given these parameters, the distribution
    computes its log probability.

    @author Samy Bengio (bengio@idiap.ch)
*/
class DistrMachine : public Distribution
{
  public:
    /// the output distribution
    Distribution *distribution;

    /// the machine that produces the parameters of distribution
    GradientMachine *machine;

    /// to interface the input
    List input_machine;

    /// to keep the derivative of the parameters for the machine alpha
    real *der_params_distribution;
    
    ///
    DistrMachine(Distribution* distribution_,GradientMachine *machine_);

    virtual void reset();
    virtual int numberOfParams();
    virtual void allocateMemory();
    virtual void freeMemory();
    virtual void loadFILE(FILE *file);
    virtual void saveFILE(FILE *file);


    virtual real frameLogProbability(real *observations, real *inputs, int t);
    virtual void iterInitialize();
    virtual void sequenceInitialize(List* inputs);
    virtual void frameBackward(real *observations, real *alpha, real *inputs, int t);
    virtual void frameExpectation(real *observations, real *inputs, int t);

    virtual ~DistrMachine();
};

#endif
