#ifndef EIGEN_DIAGONAL_GMM_INC
#define EIGEN_DIAGONAL_GMM_INC

#include "DiagonalGMM.h"
#include "SeqDataSet.h"
#include "EMTrainer.h"

/** This class can be used to model Diagonal Gaussian Mixture Models.
    They can be trained using either EM (with EMTrainer) or gradient descent
    (with GMTrainer).

    @author Samy Bengio (bengio@idiap.ch)
*/
class EigenDiagonalGMM : public DiagonalGMM
{
  public:
		real* w;
		real** eigen_vect;
		int eigen_dimension;
		real** Q;
		real* v;
		real*** pre_v;
		real*** pre_Q;
		/// The prior distribution used in MAP
		DiagonalGMM* prior_distribution;
		 /// The weight to give to the prior parameters during update
		real weight_on_prior;

    ///
    EigenDiagonalGMM(int n_observations_, int n_gaussians_, real* var_threshold_, real prior_weights_,real** eigen_vect_,int eigen_dimension_,real weight_on_prior_,DiagonalGMM* prior_distribution_);

    virtual void reset();
    virtual int numberOfParams();
    virtual void allocateMemory();
    virtual void freeMemory();

    virtual void frameEMAccPosteriors(real *observations, real log_posterior, real *inputs, int t);
    virtual void eMUpdate();

    virtual ~EigenDiagonalGMM();
};

#endif
