#ifndef EXPECTATION_MEASURER_INC
#define EXPECTATION_MEASURER_INC

#include "Measurer.h"
#include "Distribution.h"
#include "SeqDataSet.h"

/** This measurers can be used to compute and print $E[y|x]$ the expected
    value of $y$ given $x$, using a given #Distribution#.

    @author Samy Bengio (bengio@idiap.ch)
*/
class ExpectationMeasurer : public Measurer
{
  public:
    /// The expectation vector is kept here
    real *expect;
    /// the dataset on which expectations will be computed
    SeqDataSet* sdata;
    /// the distribution for which ones computes the expectation
    Distribution* distribution;

    ///
    ExpectationMeasurer(SeqDataSet *data_, Distribution* distribution_, FILE *file_);
    ///
    ExpectationMeasurer(SeqDataSet *data_, Distribution* distribution_, const char *filename);

    virtual void init();
    virtual void reset();
    virtual void measureEx();
    virtual void measureIter();

    virtual ~ExpectationMeasurer();
};

#endif
