#include "FastaSeqDataSet.h"
#include "String.h"

FastaSeqDataSet::FastaSeqDataSet(char* file, int max_load)
{

  // first read the symbol table
  createSymbolTable(file);

  // then compute the number of examples
  char command[1000];
  sprintf(command, "grep '>' %s | wc -l", file);
  FILE* f = popen(command,"r");

  if(!f) 
    error("cannot open file %s", file);
  int n;
  fscanf(f, "%d", &n);
  n_real_examples = max_load > 0 && max_load < n ? max_load : n;
  fclose(f);
  
  examples = (SeqExample*)xalloc(sizeof(SeqExample) * n_real_examples);
  n_observations = 1;
  n_inputs = 0;
  n_targets = 0;
 
  // put the file name on the object
  file_names = (char**) xalloc(sizeof(char*));
  n_file_names = 1;
  char* start = baseName(file);
  file_names[0] = (char*)xalloc(sizeof(char)*(strlen(start)+1));
  strcpy(file_names[0],start);
  
  // then read the file in FASTA format
  f = fopen(file, "r");
  if(!f)
    error("cannot open file %s", file);

  char line[1000];
  fgets(line, 1000, f); // line starting with ">" contains the name
  for(int i = 0;i < n_real_examples;i++)
  {
    sscanf(line, "%s", command);// the name is the first string
    examples[i].name = (char*)xalloc(sizeof(char)*(strlen(command)+1));
    strcpy(examples[i].name,command);
    // next lines contain the sequence
    fgets(line, 1000, f); // line containing a part of the sequence
    int old_n_frames = 0;
    n_frames = strlen(line) - 1;
    examples[i].n_real_frames = n_frames;
    examples[i].n_frames = n_frames;
    examples[i].selected_frames = NULL;
    examples[i].n_seqtargets = 0;
    examples[i].seqtargets = NULL;
    examples[i].inputs = NULL;
    examples[i].n_alignments = 0;
    examples[i].alignment = NULL;
    examples[i].observations =(real**)xalloc(sizeof(real*)*n_frames);
    while (!feof(f) && !strstr(line,">")) {
      for(int j = 0;j < n_frames;j++) {
        examples[i].observations[old_n_frames+j] = (real*)xalloc(sizeof(real));
        int index = findSymbol(line[j]);
        if (index < 0)
          error("symbol %c is not in the symbol table!",line[j]); 
        examples[i].observations[old_n_frames+j][0] = (real)index;
      }
      fgets(line, 1000, f); // line containing a part of the sequence
      if (!feof(f) && !strstr(line,">")) {
        old_n_frames += n_frames;
        n_frames = strlen(line) - 1;
        examples[i].observations =(real**)xrealloc(examples[i].observations,
          sizeof(real*)*(n_frames+old_n_frames));
      }
    }
  }
  fclose(f);
}

int FastaSeqDataSet::findSymbol(char symbol)
{
  int index = -1;
  for (int i=0;i<n_symbols;i++) {
    if (symbol == symbols[i]) {
      index = i;
      break;
    }
  }
  return index;
}

void FastaSeqDataSet::createSymbolTable(char* file)
{
  FILE *f=fopen(file,"r");
  if(!f)
    error("cannot open file %s", file);
  n_symbols = 0;
  symbols = NULL;
  char line[1000];
  fgets(line, 1000, f); // read a line
  while (!feof(f)) {
    if (!strstr(line,">")) { // line starting with ">" are not considered
      int len = strlen(line) - 1;
      for (int i=0;i<len;i++) {
        int index = findSymbol(line[i]);
        if (index < 0) {
          n_symbols++;
          if (n_symbols ==  1) {
            symbols = (char*)xalloc(sizeof(char));
          } else {
            symbols = (char*)xrealloc(symbols,sizeof(char)*n_symbols);
          }
          symbols[n_symbols-1] = line[i];
        }
      }
    }
    fgets(line, 1000, f); // next line
  }
  fclose(f);
}

FastaSeqDataSet::~FastaSeqDataSet()
{
  freeMemory();
}

void FastaSeqDataSet::freeMemory()
{

  if (n_symbols>0)
    free(symbols);

  for(int i = 0;i < n_real_examples;i++)
  {
    if(examples[i].n_real_frames > 0) 
    {
      if(n_observations > 0) 
      {
        for(int j = 0;j < examples[i].n_real_frames;j++)
          free(examples[i].observations[j]);
        free(examples[i].observations);
      }
    }
    if(examples[i].selected_frames) 
    {
      free(examples[i].selected_frames);
      examples[i].selected_frames = NULL;
    }
    if (examples[i].name) {
      free(examples[i].name);
      examples[i].name = NULL;
    }
  }
  
  for (int i = 0;i<n_file_names;i++){
     if(file_names[i] != NULL){
	free(file_names[i]);
	file_names[i] = NULL;
     }
  }
  free(file_names);
  free(examples);
}
