#ifndef FASTA_SEQ_DATASET_INC
#define FASTA_SEQ_DATASET_INC

#include "SeqDataSet.h"

/** 
   This class is used to read seqdataset in bio Fasta format.
   The format is the following: every thing is in one ascii file.
   Each sequence is preceded by a comment line where the first
   string is taken as the name of the sequence. The following lines
   contain the sequence itself, where each character is a symbol
   of the sequence (such as 'A', 'C', 'T', 'G').
  
   @author Samy Bengio (bengio@idiap.ch)
*/
class FastaSeqDataSet : public SeqDataSet
{
  public:

    /// the unique symbol mapping
    char* symbols;

    /// the number of different symbols
    int n_symbols;

    ///
    FastaSeqDataSet( char* file, int max_load=-1 );

    /// find the different symbols in the database and create the symbol table
    virtual void createSymbolTable(char* file);

    /// verify if a given symbol already exists in the symbol table
    virtual int findSymbol(char symbol);

    /// 
    virtual ~FastaSeqDataSet();

    virtual void freeMemory();
    
};
#endif

