#include "HKmeans.h"
#include "log_add.h"

HKmeans::HKmeans(DiagonalGMM *distribution_, SeqDataSet *data_, real* max_perturb_) : EMTrainer(distribution_,data_)
{
  gmm = distribution_;
  max_perturb = max_perturb_;
  max_K = gmm->n_gaussians;;
}

void HKmeans::train(List *measurers)
{

  gmm->setNGaussians(1);
  gmm->reset();
  for (int i=1;i<=max_K;i++) {
    gmm->setNGaussians(i);
    EMTrainer::train(measurers);
    if (i != max_K) {
      // then find the kernel to "split": the largest variance
      int largest_j = 0;
      real largest_var = 0;
      for (int j=0;j<i;j++) {
        real sum_var = 0;
        real* var_j = gmm->var[j];
        for (int k=0;k<gmm->n_observations;k++) {
          sum_var += *var_j * *var_j++;
        }
        if (sum_var > largest_var) {
          largest_var = sum_var;
          largest_j = j;
        }
      }
      // add a new kernel as a copy of "largest_j"
      for (int k=0;k<gmm->n_observations;k++) {
        gmm->means[i][k] = gmm->means[largest_j][k];
        gmm->var[i][k] = gmm->var[largest_j][k];
      }
      // now perturbate both the largest_j and the i means
      for (int k=0;k<gmm->n_observations;k++) {
        real perturb = bounded_uniform(-max_perturb[k],max_perturb[k]);
        gmm->means[i][k] += perturb;
        gmm->means[largest_j][k] -= perturb;
      }
      // reweight the weights
      gmm->log_weights[i] = gmm->log_weights[largest_j] - log(2.);
      gmm->log_weights[largest_j] = gmm->log_weights[i];
    }
  }
}

HKmeans::~HKmeans()
{
}
