#ifndef HTK_FILE_DATA_SET_INC
#define HTK_FILE_DATA_SET_INC

#include "StdDataSet.h"
#include "IOHtk.h"
#include "Distribution.h"

/** Create a #DataSet# from a disk file in HTK format.

    @author Samy Bengio (bengio@idiap.ch)
 */
class HtkFileDataSet : public StdDataSet
{
  public:
    int n_files;
    IOHtk** htk;
    int window_size;

    //-----

    /** Load #file# in memory.
        \begin{itemize}
          \item #window_size# gives the number of frames per input.
          \item if #max_load# > 0, it loads only the first #max_load# examples.
        \end{itemize}
    */
    HtkFileDataSet(char *file, int window_size_ = 1, int max_load=-1);
    HtkFileDataSet(char **files, int n_files_, int window_size_ = 1, int max_load=-1);

    virtual void prepareData(bool* input_to_del=NULL);

    virtual void normalizeUsingDataSet(StdDataSet *data_norm);
    virtual void normalize();
    virtual void init();
		virtual int removeUnlikelyFrames(Distribution* likely_distr, Distribution* unlikely_distr,bool* mask);
    virtual void createMaskFromParam(bool* mask);

    /// Load the normalization parameters
    virtual void loadFILE(FILE *file);
    /// Save the normalization parameters
    virtual void saveFILE(FILE *file);

    virtual ~HtkFileDataSet();
};

#endif

