#ifndef HTK_SEQ_DATASET_INC
#define HTK_SEQ_DATASET_INC

#include "SeqDataSet.h"
#include "IOHtk.h"
#include "Dictionary.h"

/** This class is used to read Htk seqdatasets
 *
 * @author Samy Bengio (bengio@idiap.ch)
 * @author Johnny Mariethoz (Johnny.Mariethoz@idiap.ch)
 */
class HtkSeqDataSet : public SeqDataSet
{
  public:
    
    /// the files, in htk format
    IOHtk** htk;

    /// used to read targets and alignments
    Dictionary* dict;

    /// number of miliseconds per frame (used during alignment)
    int n_per_frame;

    ///
    HtkSeqDataSet(char* file, int max_load=-1);
    ///
    HtkSeqDataSet(char** files,int n_files, int max_load=-1);

    virtual ~HtkSeqDataSet();
    
    
    /** destroys data allocated by #createData# 
     */
    virtual void freeMemory();

    void setDictionary(Dictionary* dict);
    void setNPerFrame(int n_per_frame);
    virtual void readTargets(char* file);
    virtual void readAlignments(char* file, bool needs_all_examples = false);
    int findExample(char* name);
    virtual void prepareData();
    virtual void createMaskFromParam(bool* mask);
    
};
#endif


