#ifndef IOBIN_INC
#define IOBIN_INC

#include "Object.h"

/// HTK File Header 
struct HTKhdr {              
    long  nSamples;
    long  sampPeriod;
    short sampSize;
    short sampKind;
};

#define BASEMASK  077         /* Mask to remove qualifiers */
#define HASENERGY  0100       /* _E log energy included */
#define HASNULLE   0200       /* _N absolute energy suppressed */
#define HASDELTA   0400       /* _D delta coef appended */
#define HASACCS   01000       /* _A acceleration coefs appended */
#define HASCOMPX  02000       /* _C is compressed */
#define HASZEROM  04000       /* _Z zero meaned */
#define HASCRCC  010000       /* _K has CRC check */
#define HASZEROC 020000       /* _0 0'th Cepstra included */
#define HASVQ    040000       /* _V has VQ index attached */


enum _BaseParmKind{
  WAVEFORM,            /* Raw speech waveform (handled by HWave) */
  LPC,LPREFC,LPCEPSTRA,LPDELCEP,   /* LP-based Coefficients */
  IREFC,                           /* Ref Coef in 16 bit form */
  MFCC,                            /* Mel-Freq Cepstra */
  FBANK,                           /* Log Filter Bank */
  MELSPEC,                         /* Mel-Freq Spectrum (Linear) */
  USER,                            /* Arbitrary user specified data */
  DISCRETE,                        /* Discrete VQ symbols (shorts) */
  ANON
};

typedef short ParmKind;          /* BaseParmKind + Qualifiers */


/** This class is used to read Htk seqdatasets
 *
 * @author Samy Bengio (bengio@idiap.ch)
 * @author Johnny Mariethoz (Johnny.Mariethoz@idiap.ch)
 */
class IOBin : public Object
{
  public:
    
    /// the data itself
    real* data;
		///target for each data
		int* target;

    /// number of columns in the data
    int n_cols;
    /// number of lines in the data
    int n_lines;

    IOBin(char* data_file,char* target_file,int n_cols_,int max_load=-1);
    virtual ~IOBin();
    
};
#endif


