#ifndef MAP_DIAGONAL_GMM_INC
#define MAP_DIAGONAL_GMM_INC

#include "DiagonalGMM.h"

/** This class is a special case of a DiagonalGMM that implements the
    MAP algorithm instead of the EM algorithm. This means that the
    mean parameters will be changed according to the Maximum A Posteriori
    algorithm, given a prior value of the means (through a prior DiagonalGMM
    given in the constructor). Moreover, the variances and weights are
    not changed, as experimental results tend to show that there is no
    effects when they are changed.

    @author Samy Bengio (bengio@idiap.ch)
    @author Johnny Mariethoz (Johnny.Mariethoz@idiap.ch)
*/
class MAPDiagonalGMM : public DiagonalGMM
{
  public:
    /// The prior distribution used in MAP
    DiagonalGMM* prior_distribution;
    /// The weight to give to the prior parameters during update
    real weight_on_prior;
		///to update an gaussian his weight should be greather than this threshold
		real thrd;

    ///
    MAPDiagonalGMM(int n_observations_, int n_gaussians_, real* var_threshold_, real prior_weights_,DiagonalGMM* prior_distribution_,real weight_on_prior_i,real thrd_=0);

    virtual void reset();

    virtual void eMIterInitialize();
    virtual void iterInitialize();

    virtual void frameViterbiAccPosteriors(real *observations, real log_posterior, real *inputs, int t);
    virtual void frameEMAccPosteriors(real *observations, real log_posterior, real *inputs, int t);
    virtual void eMUpdate();

    virtual void frameBackward(real *observations, real *alpha, real *inputs, int t);

    virtual ~MAPDiagonalGMM();
};

#endif
