
#include "MAPHMM.h"
#include "log_add.h"
#include "random.h"

MAPHMM::MAPHMM(int n_states_, Distribution **states_, real prior_transitions_,SeqDataSet* data_,real** transitions,HMM* prior_distribution_,real weight_on_prior_) : 
   HMM(n_states_,states_,prior_transitions_,data_,transitions){
  prior_distribution = prior_distribution_;
  weight_on_prior = weight_on_prior_;
  log_weight_on_prior = log(weight_on_prior);
  log_1_weight_on_prior = log(1-weight_on_prior);
 
}


void MAPHMM::eMUpdate()
{
  // first the states
  for (int i=1;i<n_states-1;i++) {
    states[i]->eMUpdate();
  }
  // then the transitions;
  for (int i=0;i<n_states-1;i++) {
    real sum_trans_acc = 0;
    for (int j=0;j<n_states;j++) {
      if (log_transitions[j][i] == LOG_ZERO)
        continue;
      sum_trans_acc += transitions_acc[j][i];
    }
    real log_sum = log(sum_trans_acc);
    for (int j=0;j<n_states;j++) {
      if (log_transitions[j][i] == LOG_ZERO)
        continue;
      log_transitions[j][i] = log_add(log_weight_on_prior + prior_distribution->log_transitions[j][i],
	                              log_1_weight_on_prior + log(transitions_acc[j][i]) - log_sum);
    }
  }
}

MAPHMM::~MAPHMM()
{
}
