#include "MLLRDiagonalGMM.h"
#include "log_add.h"
#include "random.h"
#include "matrix.h"
#include "RegTree.h"

	MLLRDiagonalGMM::MLLRDiagonalGMM(int n_observations_, int n_gaussians_, real* var_threshold_, real prior_weights_, DiagonalGMM* prior_distribution_,int n_obs,SeqDataSet* data_,real weight_on_prior_) 
: DiagonalGMM(n_observations_,n_gaussians_,var_threshold_,prior_weights_)
{
	prior_distribution = prior_distribution_;
	n_obs_per_gauss = n_obs;
	data = data_;
	shared_W = NULL;
	matrix = NULL;
	n_matrix = 0;
	inv_cov = NULL;
	ext_means = NULL;
	weight_on_prior = weight_on_prior_;
}

void MLLRDiagonalGMM::allocateMemory(){
	DiagonalGMM::allocateMemory();
	shared_W = (int*)xalloc(n_gaussians*sizeof(int));
	ext_means = (real**)xalloc(n_gaussians*sizeof(real*));
	inv_cov = (real**)xalloc(n_gaussians*sizeof(real*));
	for(int i=0;i<n_gaussians;i++){
		ext_means[i] = (real*)xalloc((1+n_observations)*sizeof(real));
		ext_means[i][0] = 1;
		inv_cov[i] = (real*)xalloc(n_observations*sizeof(real));
	}
}


void MLLRDiagonalGMM::freeMemory(){
	DiagonalGMM::freeMemory();
	if(shared_W){
		free(shared_W);
		shared_W = NULL;
	}
	if(ext_means){
		for(int i=0;i<n_gaussians;i++){
			free(ext_means[i]);
			ext_means[i] = NULL;
		}
		free(ext_means);
		ext_means = NULL;
	}
	if(inv_cov){
		for(int i=0;i<n_gaussians;i++){
			free(inv_cov[i]);
			inv_cov[i] = NULL;
		}
		free(inv_cov);
		inv_cov = NULL;
	}
	freeMatrix();
}
void MLLRDiagonalGMM::findNObsPerGaussians(int* gauss_n_obs_){
	for(int i=0;i<data->n_examples;i++){
		data->setExample(i);
		SeqExample* ex = (SeqExample*)data->inputs->ptr;
		for(int j=0;j<ex->n_frames;j++){
			real max = -100000.0;
			int i_max = 0;
			real res = max;
			for(int k=0;k<n_gaussians;k++){
				res = frameLogProbabilityOneGaussian(ex->observations[j],NULL,k);
				if(res > max){
					max = res;
					i_max = k;
				}
			}
			gauss_n_obs_[i_max]++;
		}
	}
}

void MLLRDiagonalGMM::multTransVect(real* vect,real** res,int size){
	//vector
	real* v = vect;
	//vector Transpose
	real** l_r = res;
	for(int i=0;i<size;i++,v++,l_r++){
		real* t_v = vect;
		real* c_r = *l_r;
		for(int j=0;j<size;j++)
			*c_r++ = *v * *t_v++;
	}
}
void MLLRDiagonalGMM::reset()
{
	// here, initialize the parameters to the parameters of the prior
	// distribution
	copyList(params,prior_distribution->params);
	eMIterInitialize();

	RegTree tree(this,n_obs_per_gauss);
	n_matrix = tree.buildRegTree();
	//for(int i=0;i<n_gaussians;i++)
		//message("Gauss: %d: matrix: %d",i,shared_W[i]);
	freeMatrix();

	//allocate memory
	matrix = (TransformMatrix*) xalloc(n_matrix*sizeof(TransformMatrix));
	TransformMatrix* mat = matrix;
	for(int i=0;i<n_matrix;i++,mat++){
		mat->n_gauss = 0;
		int tot_n_gauss = 0;
		//find the number of gaussians
		for(int j=0;j<n_gaussians;j++)
			if(shared_W[j] == i)
				tot_n_gauss++;	
		mat->gauss_list = (int*) xalloc(tot_n_gauss*sizeof(int));
		for(int j=0;j<n_gaussians;j++)
			if(shared_W[j] == i)
				mat->gauss_list[mat->n_gauss++] = j;

		mat->v = (real**) xalloc(mat->n_gauss*sizeof(real*));
		mat->D = (real***) xalloc(mat->n_gauss*sizeof(real**));
		for(int j=0;j<mat->n_gauss;j++){
			mat->v[j] = (real*) xalloc(n_observations*sizeof(real));
			mat->D[j] = (real**) xalloc((n_observations+1)*sizeof(real*));
			for(int k=0;k<n_observations+1;k++)
				mat->D[j][k] = (real*) xalloc((n_observations+1)*sizeof(real));
		}
		mat->Z = (real**) xalloc(n_observations*sizeof(real*));
		mat->W = (real**) xalloc(n_observations*sizeof(real*));
		mat->G = (real**) xalloc((n_observations+1)*sizeof(real*));
		for(int j=0;j<n_observations;j++){
			mat->Z[j] = (real*) xalloc((n_observations+1)*sizeof(real));
			mat->W[j] = (real*) xalloc((n_observations+1)*sizeof(real));
		}
		for(int j=0;j<n_observations+1;j++){
			mat->G[j] = (real*) xalloc((n_observations+1)*sizeof(real));
		}
	}

	for(int i=0;i<n_gaussians;i++){
		real* ext_means_i = ext_means[i];
		ext_means_i++;
		real* p_means_i = prior_distribution->means[i];
		real* inv_cov_i = inv_cov[i];
		real* p_var_i = prior_distribution->var[i];
		for(int j=0;j<n_observations;j++){
			*ext_means_i++ = *p_means_i++;
			*inv_cov_i++ = 1/ *(p_var_i++);
		}
	}
	for(int i=0;i<n_matrix;i++)
		for(int j=0;j<matrix[i].n_gauss;j++)
			multTransVect(ext_means[matrix[i].gauss_list[j]],matrix[i].D[j],n_observations+1);
		

}

void MLLRDiagonalGMM::freeMatrix(){
	if(matrix){
		TransformMatrix* mat = matrix;
		for(int i=0;i<n_matrix;i++,mat++){
			for(int j=0;j<mat->n_gauss;j++){
				free(mat->v[j]);
				mat->v[j] = NULL;
				for(int k=0;k<n_observations+1;k++){
					free(mat->D[j][k]);
					mat->D[j][k] = NULL;
				}
				free(mat->D[j]);
				mat->D[j] = NULL;
			}
			for(int j=0;j<n_observations;j++){
				if(mat->Z[j]){
					free(mat->Z[j]);
					mat->Z[j] = NULL;
				}
				if(mat->W[j]){
					free(mat->W[j]);
					mat->W[j] = NULL;
				}
			}
			for(int j=0;j<n_observations+1;j++){
				if(mat->G[j]){
					free(mat->G[j]);
					mat->G[j] = NULL;
				}
			}
			if(mat->Z){
				free(mat->Z);
				mat->Z = NULL;
			}
			if(mat->W){
				free(mat->W);
				mat->W = NULL;
			}
			if(mat->G){
				free(mat->G);
				mat->G = NULL;
			}
			if(mat->v){
				free(mat->v);
				mat->v = NULL;
			}
			if(mat->D){
				free(mat->D);
				mat->D = NULL;
			}
			if(mat->gauss_list){
				free(mat->gauss_list);
				mat->gauss_list = NULL;
			}

		}
	}
	if(matrix){
		free(matrix);
		matrix = NULL;
	}
}

void MLLRDiagonalGMM::frameEMAccPosteriors(real *observations, real log_posterior, real *inputs, int t)
{
	TransformMatrix* mat = matrix;

	for (int i=0;i<n_matrix;i++,mat++) {
		for(int j=0;j<mat->n_gauss;j++){
			int gauss_index = mat->gauss_list[j];
			real post_i = exp(log_posterior + log_weights[gauss_index] + log_probabilities_g[t][gauss_index] - log_probabilities[t]);
			real* v = mat->v[j];
			real* inv_cov_i = inv_cov[gauss_index];
			real** z_l = mat->Z;
			real* obs = observations;
			for(int k=0;k<n_observations;k++,z_l++,obs++,v++){
				real tmp =  post_i * *inv_cov_i++;
				*v += tmp;
				real* z_c = *z_l;
				real* t_ext_means = ext_means[gauss_index];
				for(int l=0;l<n_observations+1;l++){
					*z_c++ += tmp * *obs * *t_ext_means++;
				}
			}
		}
	}
}

void MLLRDiagonalGMM::eMUpdate()
{
	TransformMatrix* mat = matrix;
	Perm* pivot = new Perm(n_observations+1);

	for (int i=0;i<n_matrix;i++,mat++) {
		real**z_j = mat->Z;
		real**w_j = mat->W;
		for(int j=0;j<n_observations;j++){
			resetMat(mat->G,n_observations+1,n_observations+1);
			for(int k=0;k<mat->n_gauss;k++){
				//matrix G_j
				real** g_l = mat->G;
				real* v_k = mat->v[k];
				real** d_l = mat->D[k];
				for(int l=0;l<n_observations+1;l++){
					real* g_m = *g_l++;
					real* d_m = *d_l++;
					for(int m=0;m<n_observations+1;m++){
						*g_m++ += v_k[j] * *d_m++;
					}
				}
			}
			//end matrix G_j
			Mat* m_gi = new Mat(mat->G,n_observations+1,n_observations+1);
			Vec* v_zi = new Vec(*z_j++,n_observations+1);
			Vec* v_wi = new Vec(*w_j++,n_observations+1);
			mxLUFactor(m_gi, pivot);
			mxLUSolve (m_gi, pivot, v_zi,v_wi);
			delete (m_gi);
			delete (v_zi);
			delete (v_wi);
		}
		//update means
		Mat* m_w = new Mat(mat->W,n_observations,n_observations+1);
		 
		for(int j=0;j<mat->n_gauss;j++){
			int gauss_index = mat->gauss_list[j];
			Vec* v_ext_means = new Vec(ext_means[gauss_index],n_observations+1);
			Vec* v_means = new Vec(means[gauss_index],n_observations);
			mxMatMulVec (m_w, v_ext_means,v_means);
				real* means_i = means[gauss_index];
				real* p_means_i = prior_distribution->means[gauss_index];
				for(int i=0;i<n_observations;i++){
				*means_i = *p_means_i++ * weight_on_prior + (1-weight_on_prior) * *means_i++;
				}
			delete (v_means);
			delete(v_ext_means);
		}
		delete(m_w);
	}
	delete (pivot);
}
void MLLRDiagonalGMM::resetMat(real** mat,int n_line,int n_col){
	
	real** line = mat;
	
	for(int i=0;i<n_line;i++){
		real* col = *line++;
		for(int j=0;j<n_col;j++){
			*col++ = 0.0;
		}
	}

}
void MLLRDiagonalGMM::eMIterInitialize()
{
	// initialize the accumulators to 0 and compute pre-computed value
	
	TransformMatrix* mat = matrix;
	for(int i=0;i<n_matrix;i++,mat++){
		resetMat(mat->Z,n_observations,n_observations+1);
		real** v_j = mat->v;
		for(int j=0;j<mat->n_gauss;j++){
			real* v_k = *v_j++;
			for(int k=0;k<n_observations;k++)
			*v_k++ = 0.0;
		}
	}
	//pre-computed values	
	for (int i=0;i<n_gaussians;i++) {
		real *v = var[i];
		real *sum = &sum_log_var_plus_n_obs_log_2_pi[i];
		*sum = n_observations * LOG_2_PI;
		real *mh_i = minus_half_over_var[i];
		for (int j=0;j<n_observations;j++) {
			*mh_i++ = -0.5 / *v;
			*sum += log(*v++);
		}
		*sum *= -0.5;
	}
}

void MLLRDiagonalGMM::iterInitialize()
{
	for (int i=0;i<n_gaussians;i++) {
		real* means_prior_i = prior_distribution->means[i];
		real* means_i = means[i];
		for (int j=0;j<n_observations;j++,means_i++,means_prior_i++) {
			//*means_i = weight_on_prior * *means_prior_i +
			//    (1 - weight_on_prior) * *means_i;
			*means_i = *means_prior_i;
		}
	}
}


MLLRDiagonalGMM::~MLLRDiagonalGMM()
{
	freeMemory();
}
