#ifndef MLLR_DIAGONAL_GMM_INC
#define MLLR_DIAGONAL_GMM_INC

#include "DiagonalGMM.h"
#include "SeqDataSet.h"

struct TransformMatrix{
	real** Z;
	real** G;
	real** v;
	real** W;
	real*** D;
	int n_gauss;
	int * gauss_list;
};


/** This class is a special case of a DiagonalGMM that implements the
    MLLR algorithm instead of the EM algorithm. This means that the
    mean parameters will be changed according to the MLLR
    algorithm, given a prior value of the means (through a prior DiagonalGMM
    given in the constructor). Moreover, the variances and weights are
    not changed, as experimental results tend to show that there is no
    effects when they are changed.

    ATTENTION: this algorithm needs only ONE EM iteration!

    @author Samy Bengio (bengio@idiap.ch)
    @author Johnny Mariethoz (Johnny.Mariethoz@idiap.ch)
*/
class MLLRDiagonalGMM : public DiagonalGMM
{
  public:
    /// The prior distribution used in MLLR
    DiagonalGMM* prior_distribution;

    TransformMatrix* matrix;
		real weight_on_prior;
    int n_matrix;
    int* shared_W;
    int n_obs_per_gauss;
    SeqDataSet* data;
		real** ext_means;
		real** inv_cov;

    ///
    MLLRDiagonalGMM(int n_observations_, int n_gaussians_, real* var_threshold_, real prior_weights_,DiagonalGMM* prior_distribution_,int n_obs,SeqDataSet* data_,real weight_on_prior_);
    virtual void allocateMemory();
    virtual void freeMemory();

    virtual void findNObsPerGaussians(int* gauss_n_obs_);
    virtual void reset();
    virtual void freeMatrix();
		virtual void multTransVect(real* vect,real** res,int size);
		virtual void resetMat(real** mat,int n_line,int n_col);

    virtual void eMIterInitialize();
    virtual void iterInitialize();

    virtual void frameEMAccPosteriors(real *observations, real log_posterior, real *inputs, int t);
    virtual void eMUpdate();

    virtual ~MLLRDiagonalGMM();
};

#endif
