#include "MultiCriterion.h"

MultiCriterion::MultiCriterion(int n_criterions_,Criterion** criterions_,real* weights_)
{
  n_criterions = n_criterions_;
  criterions = criterions_;
  n_inputs = criterions[0]->n_inputs;
  weights = weights_;
  if (weights) {
    allocate_weights = false;
  } else {
    allocate_weights = true;
    weights = (real*)xalloc(sizeof(real)*n_criterions);
    for (int i=0;i<n_criterions;i++)
      weights[i] = 1.;
  }
}

void MultiCriterion::setDataSet(DataSet *data_)
{
  data = data_;
  for (int i=0;i<n_criterions;i++)
    criterions[i]->setDataSet(data_);
}

void MultiCriterion::iterInitialize()
{
  for (int i=0;i<n_criterions;i++)
    criterions[i]->iterInitialize();
}

void MultiCriterion::reset()
{
  for (int i=0;i<n_criterions;i++)
    criterions[i]->reset();
}

void MultiCriterion::init()
{
  GradientMachine::init();
  for (int i=0;i<n_criterions;i++)
    criterions[i]->init();
}

void MultiCriterion::forward(List *inputs)
{
  real *ptr_out = (real *)outputs->ptr;
  *ptr_out = 0.;
  real *w = weights;
  for (int i=0;i<n_criterions;i++) {
    criterions[i]->forward(inputs);
    *ptr_out += *((real*)criterions[i]->outputs->ptr) * *w++;
  }
}

void MultiCriterion::backward(List *inputs, real *alpha)
{
  List* in = inputs;
  real *b = beta;
  while (in) {
    for (int i=0;i<in->n;i++)
      *b++ = 0;
    in = in->next;
  }
  real *w = weights;
  for (int i=0;i<n_criterions;i++,w++) {
    criterions[i]->backward(inputs,alpha);
    in = inputs;
    b = beta;
    real* bi = criterions[i]->beta;
    while (in) {
      for (int j=0;j<in->n;j++)
        *b++ += *bi++ * *w;
      in = in->next;
    }
  }
}

MultiCriterion::~MultiCriterion()
{
  if (allocate_weights)
    free(weights);
}
