#include "RegTree.h"

RegTree::RegTree(MLLRDiagonalGMM* gmm_,int n_obs_per_gauss_){
	gmm = gmm_;
	n_obs_per_gauss = n_obs_per_gauss_;
	right_center = (real*)xalloc(sizeof(real)*gmm->n_observations);
	left_center = (real*)xalloc(sizeof(real)*gmm->n_observations);
	sigma = (real*)xalloc(sizeof(real)*gmm->n_observations);
	delta = (real*)xalloc(sizeof(real)*gmm->n_observations);
	gauss_list = (int*)xalloc(gmm->n_gaussians*sizeof(int));
	n_gauss = 0;
	n_matrix_shared = 0;
	gauss_n_obs = (int*)xcalloc(gmm->n_gaussians,sizeof(int));
	perturb = 0.000001;

}

RegTree::~RegTree(){

	if(left_center){
		free(left_center);
		left_center = NULL;
	}
	if(right_center){
		free(right_center);
		right_center = NULL;
	}
	if(sigma){
		free(sigma);
		sigma = NULL;
	}
	if(delta){
		free(delta);
		delta = NULL;
	}
	if(gauss_list){
		free(gauss_list);
		gauss_list = NULL;
	}

	if(gauss_n_obs){
		free(gauss_n_obs);
		gauss_n_obs = NULL;
	}
}


int RegTree::buildRegTree(){
	printf("MLLR: build tree");

	for(int i=0;i<gmm->n_gaussians;i++)
		gauss_list[i] = i;
	n_gauss = gmm->n_gaussians;
	computeMeanSigma();
	gmm->findNObsPerGaussians(gauss_n_obs);
	for(int i=0;i<gmm->n_gaussians;i++){
		gmm->shared_W[i] = -1;
	}
	int sum = 0;
	for(int i=0;i<gmm->n_gaussians;i++){
		sum += gauss_n_obs[i];
	}
	if(sum < n_obs_per_gauss){
		for(int i=0;i<n_gauss;i++)
			gmm->shared_W[i] = 0;
		n_matrix_shared = 1;
	}
	else
	{n_matrix_shared = 0;
		buildTree();}
		printf("\n");
		return n_matrix_shared;
}

void RegTree::buildTree(){
	printf(".");
	computeMeanSigma();
	computeDelta();
	perturbCenter();
	int* right_gauss_list = (int*)xalloc(gmm->n_gaussians*sizeof(int));
	int* left_gauss_list = (int*)xalloc(gmm->n_gaussians*sizeof(int));
	int n_right_gauss = 0;
	int n_left_gauss = 0;
	//distribute
	real prev_sum_dist = INF;
	real sum_dist;
	int iter = 0;
	int max_iter = 500;
	real stop_val = 0.00001;
	while(1){
		sum_dist = 0.0;
		n_left_gauss = 0;
		n_right_gauss = 0;
		for(int i=0;i<n_gauss;i++){
			real left_dist = computeDist(gmm->means[gauss_list[i]],left_center);
			real right_dist = computeDist(gmm->means[gauss_list[i]],right_center);
			//message("dist: %.3f %.3f",left_dist,right_dist);
			if(left_dist < right_dist){
				left_gauss_list[n_left_gauss++] = gauss_list[i];
				sum_dist += left_dist;
			}else{
				right_gauss_list[n_right_gauss++] = gauss_list[i];
				sum_dist += right_dist;
			}
		}
		//message("k dist: %.9f prev: %.9f",sum_dist,prev_sum_dist);
		if (prev_sum_dist - sum_dist < stop_val) {
			print("kmeans converge in %d iterations\n",iter+1);
			break;
		}
		prev_sum_dist = sum_dist;

		iter++;
		if ((iter >= max_iter) && (max_iter > 0)) {
			print("\n");
			warning("emtrainer: you have reached the maximum number of iterations: %d",iter+1);
			break;
		}
		computeMean(left_gauss_list,n_left_gauss,left_center);
		computeMean(right_gauss_list,n_right_gauss,right_center);
	}

	/*	if((right_gmm->n_gaussians == 0.0)||(left_gmm->n_gaussians == 0.0))
			printf("tree regression class 0 observations on the left or right node\n");*/

	int right_n_obs = computeNObs(right_gauss_list,n_right_gauss);
	int left_n_obs = computeNObs(left_gauss_list,n_left_gauss);

	if(n_right_gauss > 0){
		if((n_obs_per_gauss > right_n_obs)||(n_right_gauss == 1)){
			//printf("link right with n_obs:%d\n",right_n_obs+left_n_obs);
			//link node
			//message("Link right node");
			for(int i=0;i<n_right_gauss;i++)
				gmm->shared_W[right_gauss_list[i]] = n_matrix_shared;
			n_matrix_shared++;
			free(right_gauss_list);
			right_gauss_list = NULL;

		}
		else{
			int* tmp = gauss_list;
			gauss_list = right_gauss_list;
			right_gauss_list = tmp;
			n_gauss = n_right_gauss;
			buildTree();
		}
	}

	if(n_left_gauss > 0) {
		if((n_obs_per_gauss > left_n_obs)||(n_left_gauss == 1)){
			//printf("link left with n_obs:%d\n",right_n_obs+left_n_obs);
			//linkNode();
			//message("Link left node");
			for(int i=0;i<n_left_gauss;i++)
				gmm->shared_W[left_gauss_list[i]] = n_matrix_shared;
			n_matrix_shared++;
			free(left_gauss_list);
			left_gauss_list = NULL;
		}
		else{
			int* tmp = gauss_list;
			gauss_list = left_gauss_list;
			left_gauss_list = tmp;
			n_gauss = n_left_gauss;
			buildTree();
		}
	}
	free(left_gauss_list);
	left_gauss_list = NULL;
	free(right_gauss_list);
	right_gauss_list = NULL;


}

int RegTree::computeNObs(int* g,int n){
	int sum = 0;
	for(int i=0;i<n;i++)
		sum += gauss_n_obs[g[i]];   
	return sum;
}
void RegTree::computeMean(int* g_index,int n_val,real* means){
	for(int i=0;i<gmm->n_observations;i++)
		means[i] = 0;
	for(int i=0;i<n_val;i++){
		int gauss_index = g_index[i];
		for(int j=0;j<gmm->n_observations;j++){
			means[j] += gmm->means[gauss_index][j];
		}
	}
	for(int i=0;i<gmm->n_observations;i++)
		means[i] /= n_val;
}
void RegTree::computeMeanSigma(){
	for(int j=0;j<gmm->n_observations;j++){
		left_center[j] = 0;
		sigma[j] = 0;
	}
	for(int i=0;i<n_gauss;i++){
		int gauss_index = gauss_list[i];
		for(int j=0;j<gmm->n_observations;j++){
			left_center[j] += gmm->means[gauss_index][j];
			sigma[j] +=  gmm->means[gauss_index][j] *  gmm->means[gauss_index][j];
			//message("%d %d %d %.3f %.3f\n",i,j,gauss_index,gmm->means[gauss_index][j],left_center[j]);
		}
	}
	for(int j=0;j <gmm->n_observations;j++){
		left_center[j] /= n_gauss;

		sigma[j] = sqrt(sigma[j]/n_gauss - (left_center[j]*left_center[j]));
		if(sigma[j]< 0.001)
			sigma[j] = 0.001;
	}
}

void RegTree::linkNode(){
	//message("link node");
}

void RegTree::computeDelta(){
	for(int i=0;i<gmm->n_observations;i++){
		real dist;
		dist = sigma[i] * perturb;
		delta[i] = bounded_uniform(-dist,dist);
		//message("Delta: %.6f, sigma: %.6f dist: %.6f perturb: %.6f",delta[i],sigma[i],dist,perturb);
		//message("rand:%g =>%g = %g * %g\n",delta[i],dist,sigma[i]);
	}
}

void RegTree::perturbCenter(){

	for(int i=0;i<gmm->n_observations;i++){
		right_center[i] = left_center[i];
		left_center[i] += delta[i];
		right_center[i] -= delta[i];
	}

}

real RegTree::computeDist(real* means,real* observations)

{
	real dist = 0;
	real* means_i = means;
	real *x = observations;
	for(int j = 0; j < gmm->n_observations; j++) {
		real diff = *x++ - *means_i++;
		dist += diff*diff;
	}
	return dist;
}



void RegTree::distribute(int* right_gauss_list,int& n_right_gauss,int* left_gauss_list,int& n_left_gauss){
	n_right_gauss = 0;
	n_left_gauss = 0;
	for(int i=0;i<n_gauss;i++){
		real left_dist = computeDist(gmm->means[gauss_list[i]],left_center);
		real right_dist = computeDist(gmm->means[gauss_list[i]],right_center);
		if(left_dist > right_dist)
			left_gauss_list[n_left_gauss++] = gauss_list[i];
		else
			right_gauss_list[n_right_gauss++] = gauss_list[i];
	}

}

