#include "SigmMLP.h"

SigmMLP::SigmMLP(int n_inputs_, int n_hidden_, int n_outputs_, bool is_linear_outputs_, bool inputs_to_outputs_)
{
  n_hidden = n_hidden_;
  is_linear_outputs = is_linear_outputs_;
  add_layer = NULL;
  sum_layer = NULL;
  if (n_hidden>0) {
    hidden_layer = new Linear(n_inputs_,n_hidden);
    hidden_layer->init();
    addMachine(hidden_layer);

    if (inputs_to_outputs_) {
      add_layer = new Linear(n_inputs_,n_outputs_);
      add_layer->init();
      addMachine(add_layer);
    }

    hidden_tanh_layer = new Tanh(n_hidden);
    hidden_tanh_layer->init();
    addLayer();
    addMachine(hidden_tanh_layer);
    connectOn(hidden_layer);

    addLayer();
    outputs_layer = new Linear(n_hidden,n_outputs_);
    outputs_layer->init();
    addMachine(outputs_layer);
    connectOn(hidden_tanh_layer);

    if (inputs_to_outputs_) {
      sum_layer = new SumMachine(n_outputs_,2);
      sum_layer->init();
      addLayer();
      addMachine(sum_layer);
      connectOn(add_layer);
      connectOn(outputs_layer);
    }
  } else {
    outputs_layer = new Linear(n_inputs_,n_outputs_);
    outputs_layer->init();
    addFCL(outputs_layer);
  }
  if (!is_linear_outputs) {
    outputs_sigm_layer = new Sigmoid(n_outputs_);
    outputs_sigm_layer->init();
    addFCL(outputs_sigm_layer);
  }
}

SigmMLP::~SigmMLP()
{
  if (n_hidden>0) {
    delete hidden_layer;
    delete hidden_tanh_layer;
  }
  delete outputs_layer;
  if (!is_linear_outputs)
    delete outputs_sigm_layer;
  if (add_layer)
    delete add_layer;
}


