#include "TestNllMeasurer.h"

TestNllMeasurer::TestNllMeasurer(List *inputs_, SeqDataSet *data_, FILE *file_) : Measurer(data_, file_)
{
   inputs = inputs_;
   sdata = data_;
   n_examples = data->n_real_examples;
   measure = (real*)xalloc(sizeof(real)*2*n_examples);

   n_frames = (int*)xalloc(sizeof(int)*n_examples);
   real *r_measure = (real *)measure;
   erreur = r_measure;
   erreur_c = r_measure+n_examples;

   *erreur = 0;

   reset();
}

TestNllMeasurer::TestNllMeasurer(List *inputs_, SeqDataSet *data_, const char *filename) : Measurer(data_, filename)
{
   inputs = inputs_;
   sdata = data_;
   n_examples = data->n_real_examples;
   measure = xalloc(sizeof(real)*2*n_examples);

   real *r_measure = (real *)measure;
   erreur = r_measure;
   erreur_c = r_measure+n_examples;

   *erreur = 0;

   reset();
}

void TestNllMeasurer::measureEx()
{
   erreur_c[n_examples] += *(real*)inputs->ptr;
   n_frames[n_examples] += sdata->n_frames;
   tot_n_frames += sdata->n_frames;
   n_examples++;
}

void TestNllMeasurer::measureIter()
{
   real sum = 0;
   for(int i=0;i<n_examples;i++){
      sum += erreur_c[i];
      erreur[i] = erreur_c[i] / ((real)n_frames[i]);
      fprintf(file, "%s %g\n", sdata->file_names[i],erreur[i]);
   }
   tot_erreur = sum / ((real)tot_n_frames);

   fprintf(file, "# Total: %g\n", tot_erreur);
   fflush(file);
   reset();
}

void TestNllMeasurer::init()
{
   Measurer::init();
   reset();
}

void TestNllMeasurer::reset()
{
   for(int i=0;i<n_examples;i++){
      n_frames[i] = 0;
      erreur_c[i] = 0;
   }
   tot_n_frames = 0;
   n_examples = 0;
}

TestNllMeasurer::~TestNllMeasurer()
{
   if(n_frames){
      free(n_frames);
      n_frames = NULL;
   }
   if(measure){
      free(measure);
      measure = NULL;
   }
}

