#include "TestNllSaturationMeasurer.h"

TestNllSaturationMeasurer::TestNllSaturationMeasurer(Distribution* machine_,List *inputs_, SeqDataSet *data_, FILE *file_) : Measurer(data_, file_)
{
   machine = machine_;
   inputs = inputs_;
   sdata = data_;
   n_examples = data->n_real_examples;
   measure = (real*)xalloc(sizeof(real)*2*n_examples);

   n_frames = (int*)xalloc(sizeof(int)*n_examples);
   real *r_measure = (real *)measure;
   erreur = r_measure;
   erreur_c = r_measure+n_examples;

   *erreur = 0;

   alpha[0] = 1;
   sum_derivees = (real*)xalloc(sizeof(real)*n_examples);
   sum_derivees_c = (real*)xalloc(sizeof(real)*n_examples);
   n_sum_der = (int*)xalloc(sizeof(int)*n_examples);

   reset();
}

TestNllSaturationMeasurer::TestNllSaturationMeasurer(Distribution* machine_,List *inputs_, SeqDataSet *data_, const char *filename) : Measurer(data_, filename)
{
   machine = machine_;
   inputs = inputs_;
   sdata = data_;
   n_examples = data->n_real_examples;
   measure = xalloc(sizeof(real)*2*n_examples);

   real *r_measure = (real *)measure;
   erreur = r_measure;
   erreur_c = r_measure+n_examples;

   *erreur = 0;

   alpha[0] = 1;
   sum_derivees = (real*)xalloc(sizeof(real)*n_examples);
   sum_derivees_c = (real*)xalloc(sizeof(real)*n_examples);
   n_sum_der = (int*)xalloc(sizeof(int)*n_examples);

   reset();
}

void TestNllSaturationMeasurer::measureEx()
{
  erreur_c[n_examples] += *(real*)inputs->ptr;
  n_frames[n_examples] += sdata->n_frames;
  tot_n_frames += sdata->n_frames;

  machine->backward(data->inputs,alpha);
  List *der_params_ = machine->der_params;
  while(der_params_)
  {
    real *der = (real *)der_params_->ptr;
    for(int i = 0; i < der_params_->n; i++)
    {
      sum_derivees_c[n_examples] += fabs(*der++);
      n_sum_der[n_examples]++;
    }
    der_params_ = der_params_->next;
  }
  machine->sequenceInitialize(NULL);
  n_examples++;
}

void TestNllSaturationMeasurer::measureIter()
{

   real sum = 0;
   for(int i=0;i<n_examples;i++){
      sum += erreur_c[i];
      erreur[i] = erreur_c[i] / ((real)n_frames[i]);
      sum_derivees[i] = sum_derivees_c[i] / (real)(n_sum_der[i] * n_frames[i]);
      fprintf(file, "%s %g %g\n", sdata->file_names[i],erreur[i],sum_derivees[i]);
   }
   tot_erreur = sum / ((real)tot_n_frames);

   fprintf(file, "# Total: %g\n", tot_erreur);
   fflush(file);
   reset();
}

void TestNllSaturationMeasurer::init()
{
   Measurer::init();
   reset();
}

void TestNllSaturationMeasurer::reset()
{
   for(int i=0;i<n_examples;i++){
      n_frames[i] = 0;
      erreur_c[i] = 0;
      sum_derivees[i] = 0;
      sum_derivees_c[i] = 0;
      n_sum_der[i] = 0;
   }
   tot_n_frames = 0;
   n_examples = 0;
   machine->sequenceInitialize(NULL);
}

TestNllSaturationMeasurer::~TestNllSaturationMeasurer()
{
   if(n_frames){
      free(n_frames);
      n_frames = NULL;
   }
   if(sum_derivees){
      free(sum_derivees);
      sum_derivees = NULL;
   }
   if(sum_derivees_c){
      free(sum_derivees_c);
      sum_derivees_c = NULL;
   }
   if(n_sum_der){
      free(n_sum_der);
      n_sum_der = NULL;
   }
   if(measure){
      free(measure);
      measure = NULL;
   }
}

