#ifndef TEST_NLL_SATURATION_MEASURER_INC
#define TEST_NLL_SATURATION_MEASURER_INC

#include "Measurer.h"
#include "SeqDataSet.h"
#include "Distribution.h"

/** This class measures the negative log likelihood. In fact, it supposes
    that the input given is already the negative log likelihood. It is
    then normalized by the total number of frames.

    @author Samy Bengio (bengio@idiap.ch)
*/
class TestNllSaturationMeasurer : public Measurer
{
  public:
    /// contains the average negative log likelihood for all frames
    real tot_erreur;
    /// contains the average negative log likelihood
    real *erreur;
    /// contains the total negative log likelihood
    real *erreur_c;
    /// contains a pointer to the negative log likelihood to measure
    List *inputs;
    /// local counter of the total number of frames
    int tot_n_frames;
    /// local counter of the total number of frames for each examples
    int* n_frames;
    /// total of number examples
    int n_examples;
    /// local pointer to the dataset, used to count the number of frames
    SeqDataSet *sdata;

    real *sum_derivees;
    real *sum_derivees_c;
    int *n_sum_der;
    real alpha[1];
    Distribution* machine;

    ///
    TestNllSaturationMeasurer(Distribution* machine_,List *inputs_, SeqDataSet* data_, FILE *file_);
    ///
    TestNllSaturationMeasurer(Distribution* machine_,List *inputs_, SeqDataSet* data_, const char *filename);

    virtual void init();
    virtual void reset();
    virtual void measureEx();
    virtual void measureIter();

    virtual ~TestNllSaturationMeasurer();
};

#endif
