
#include "TransposeDistribution.h"
#include "log_add.h"

TransposeDistribution::TransposeDistribution(Distribution* distribution_,int n_observations_){
   distribution = distribution_;
   n_observations = n_observations_;
   input = (List*)xalloc(sizeof(List));
   SeqExample* ex = (SeqExample*) xalloc(sizeof(SeqExample));
   if(n_observations % distribution->n_observations)
      error("TransposeDistribution: can't subdivise n_observation\n");
   ex->n_real_frames = n_observations / distribution->n_observations;
   ex->n_frames = ex->n_real_frames;
   n_inputs = distribution->n_inputs * ex->n_real_frames;
   if(n_inputs > 0)
      ex->inputs = (real**) xalloc(sizeof(real*)*ex->n_frames);
   else
      ex->inputs = NULL;
   ex->observations = (real**) xalloc(sizeof(real*)*ex->n_frames); 
   ex->name = NULL;
   ex->n_seqtargets = 0;
   ex->seqtargets = NULL;
   input->ptr = ex;
   input->n = 1;
   input->next = NULL;
   allocateMemory();

}

void TransposeDistribution::loadFILE(FILE* file){
   distribution->loadFILE(file);
}

void TransposeDistribution::saveFILE(FILE* file){
   distribution->saveFILE(file);
}

void TransposeDistribution::transpose(real* observations, real* inputs){
   SeqExample* ex = (SeqExample*)input->ptr;
   real** ex_obs = ex->observations;
   real* obs = observations;
   for(int i=0;i<ex->n_frames;i++,ex_obs++,obs+=distribution->n_observations)
      *ex_obs = obs;
   if(ex->inputs){
      real** ex_in = ex->inputs;
      real* in = inputs;
      for(int i=0;i<ex->n_frames;i++,ex_in++,in+=distribution->n_inputs)
	 *ex_in = in;
   }
   distribution->eMSequenceInitialize(input);
}

real TransposeDistribution::frameLogProbability(real *observations, real *inputs, int t)
{
  transpose(observations,inputs);
  log_probabilities[t] = distribution->logProbability(input); 
  return log_probabilities[t];
}

void TransposeDistribution::allocateMemory()
{
  max_n_frames = 1;
  log_probabilities = (real*)xalloc(sizeof(real)*max_n_frames);
}

void TransposeDistribution::iterInitialize()
{
  distribution->iterInitialize();
}

void TransposeDistribution::eMIterInitialize()
{
   distribution->eMIterInitialize();
}

void TransposeDistribution::eMSequenceInitialize(List* inputs)
{
   if (!inputs)
      return;
   SeqExample* ex = (SeqExample*)inputs->ptr;
   if (ex->n_real_frames > max_n_frames) {
      max_n_frames = ex->n_real_frames;
      log_probabilities = (real*)xrealloc(log_probabilities,sizeof(real)*max_n_frames);
   }

   distribution->eMSequenceInitialize(inputs);
}

void TransposeDistribution::frameEMAccPosteriors(real *observations, real log_posterior, real *inputs, int t)
{
  transpose(observations,inputs);
  distribution->logProbability(input);
  distribution->eMAccPosteriors(input,log_posterior); 
}

void TransposeDistribution::eMUpdate()
{
   distribution->eMUpdate();
}


void TransposeDistribution::frameBackward(real *observations, real *alpha, real *inputs, int t)
{
  transpose(observations,inputs);
  distribution->logProbability(input);
  distribution->backward(input,alpha); 
}

TransposeDistribution::~TransposeDistribution()
{
   SeqExample* ex = (SeqExample*)input->ptr;   
   if(ex->inputs){
      free(ex->inputs);
      ex->inputs = NULL;
   }
   if(ex->observations){
	free(ex->observations);
	ex->observations = NULL;
   }
   if(ex){
      free(ex);
      ex = NULL;
   }
   if(input){
      free(input);
      input = NULL;
   }
   if(log_probabilities){
      free(log_probabilities);
      log_probabilities = NULL;
   }
}
