#ifndef TRANSPOSE_DISTRIBUTION_INC
#define TRANSPOSE_DISTRIBUTION_INC

#include "Distribution.h"
#include "List.h"
#include "SeqDataSet.h"

/** This class is designed to  transpose data (vector to matrix)
    typically for HMM2.

    Note that the output of a distribution is the negative log likelihood.

    @author Johnny Mariethoz (Johnny.Mariethoz@idiap.ch)
    @author Samy Bengio (bengio@idiap.ch)
*/
class TransposeDistribution : public Distribution
{
  public:
     /// internal distribution "internal HMM" for HMM2
     Distribution* distribution;

     /// 
     List* input;
    /// 
    TransposeDistribution(Distribution* distrib_,int n_observations_);

    /// forward this methods to distribution
    void loadFILE(FILE* file);

    
    /// forward this methods to distribution
    void saveFILE(FILE* file);

    /// transpose and forward this methods to distribution
    virtual real frameLogProbability(real *observations, real *inputs, int t);

    /// transpose and forward this methods to distribution
    virtual void eMIterInitialize();

    /// transpose frame and input vector to matrix
    void transpose(real *observations, real *inputs);

    void allocateMemory();
    

    /// forward this methods to distribution
    virtual void eMSequenceInitialize(List* inputs);
    
    /// forward this methods to distribution
    virtual void iterInitialize();
   
    /// transpose and forward this methods to distribution
    virtual void frameEMAccPosteriors(real *observations, real log_posterior, real *inputs, int t);
    
    /// and forward this methods to distribution
    virtual void eMUpdate();

    /// transpose and forward this methods to distribution
    virtual void frameBackward(real *observations, real *alpha, real *inputs,int t);

    virtual ~TransposeDistribution();
};

#endif

