#include "TwoClassBoostingMeasurer.h"

TwoClassBoostingMeasurer::TwoClassBoostingMeasurer(FILE *file_) : BoostingMeasurer(file_)
{
  measure = xalloc(sizeof(real)*4);
 
  real *r_measure = (real *)measure;
  erreur = r_measure;
  erreur_c = r_measure+1;

  reset();
}

TwoClassBoostingMeasurer::TwoClassBoostingMeasurer(const char *filename) : BoostingMeasurer(filename)
{
  measure = xalloc(sizeof(real)*4);

  real *r_measure = (real *)measure;
  erreur = r_measure;
  erreur_c = r_measure+1;

  reset();
}

void TwoClassBoostingMeasurer::measureEx()
{
  real *out = (real *)inputs->ptr;
  real *des = (real *)data->targets;

  int choix = 0;
  if(fabs(*out - tabclasses[0]) > fabs(*out - tabclasses[1]))
    choix = 1;

  if(tabclasses[choix] != *des)
  {
    *erreur_c += weights[current_example];    
    status[current_example++] = -1;
  }
  else
    status[current_example++] = 1;
}

void TwoClassBoostingMeasurer::measureIter()
{
  *erreur = *erreur_c;
  beta = *erreur/(1. - *erreur);

  fprintf(file, "%g ==> %g for beta\n", *erreur, beta);
  fflush(file);
  reset();
}

void TwoClassBoostingMeasurer::setData(DataSet *data_)
{
  BoostingMeasurer::setData(data_);

  if(data->n_targets != 1)
    warning("TwoClassBoostingMeasurer: the data has %d ouputs", data->n_targets);
  
  int n_set = 0;
  for(int i = 0; i < data->n_examples; i++)
  {
    data->setExample(i);
    
    bool flag = false;
    for(int k = 0; k < n_set; k++)
    {
      if(((real *)data->targets)[0] == tabclasses[k])
        flag = true;
    }

    if(!flag)
    {
      if(n_set == 2)
        error("TwoClassBoostingMeasurer: you have more than two classes");

      tabclasses[n_set++] = ((real *)data->targets)[0];
    }
  }

  switch(n_set)
  {
    case 0:
      warning("TwoClassBoostingMeasurer: you have no examples");
      tabclasses[0] = 0;
      tabclasses[1] = tabclasses[0];
      break;
    case 1:
      warning("TwoClassBoostingMeasurer: you have only one class [%g]", tabclasses[0]);
      tabclasses[1] = tabclasses[0];
      break;
    case 2:
      message("TwoClassBoostingMeasurer: two classes detected [%g and %g]", tabclasses[0], tabclasses[1]);
      break;
  }     
}

void TwoClassBoostingMeasurer::reset()
{
  *erreur_c = 0;
  current_example = 0;
}

TwoClassBoostingMeasurer::~TwoClassBoostingMeasurer()
{
  free(measure);
}
