#include "TwoClassConfidenceBagMeasurer.h"

TwoClassConfidenceBagMeasurer::TwoClassConfidenceBagMeasurer(BaggingMachine *b_machine_, DataSet *data_, FILE *file_) : Measurer(data_,file_)
{
  measure = xalloc(sizeof(real)*2);

  data = data_;
  b_machine = b_machine_;

  reset();
}

TwoClassConfidenceBagMeasurer::TwoClassConfidenceBagMeasurer(BaggingMachine *b_machine_, DataSet *data_, const char *filename) : Measurer(data_,filename)
{
  measure = xalloc(sizeof(real)*2);

  data = data_;
  b_machine = b_machine_;

  reset();
}

void TwoClassConfidenceBagMeasurer::measureEx()
{
  int n_trainers = b_machine->n_trainers;
  int n_0 = 0;
  int n_1 = 0;

  real *out_bag = (real*)b_machine->outputs->ptr;
  for (int i=0;i<n_trainers;i++) {
    real *out_from = (real*)b_machine->trainers[i]->machine->outputs->ptr;
    if (fabs(*out_from - tabclasses[0]) > fabs(*out_from - tabclasses[1]))
      n_1++;
    else
      n_0++;
  }

  // print the output
  fprintf(file,"out ");
  for (int i=0;i<b_machine->n_outputs;i++)
    fprintf(file,"%g ",*out_bag++);

  // print the target
  real *des = (real*)data->targets;
  fprintf(file,"target ");
  for (int i=0;i<b_machine->n_outputs;i++)
    fprintf(file,"%g ",*des++);

  //print the confidence
  fprintf(file,"confidence ");
  if (n_1 > n_0)
    fprintf(file," %g%%\n",(real)n_1*100/n_trainers);
  else
    fprintf(file," %g%%\n",(real)n_0*100/n_trainers);
}

void TwoClassConfidenceBagMeasurer::measureIter()
{
}

void TwoClassConfidenceBagMeasurer::init()
{
  if(b_machine->n_outputs != 1)
    error("twoclassconfidencebagmeasurer: the machine has %d ouputs", b_machine->n_outputs);

  if(data->n_targets != 1)
    error("twoclassconfidencebagmeasurer: the data has %d ouputs", data->n_targets);

  int n_set = 0;
  for(int i = 0; i < data->n_examples; i++)
  {
    data->setExample(i);

    bool flag = false;
    for(int k = 0; k < n_set; k++)
    {
      if(*(real*)data->targets == tabclasses[k])
        flag = true;
    }

    if(!flag)
    {
      if(n_set == 2)
        error("twoclassconfidencebagmeasurer: you have more than two classes");

      tabclasses[n_set++] = *(real*)data->targets;
    }
  }

  switch(n_set)
  {
    case 0:
      warning("twoclassconfidencebagmeasurer: you have no examples");
      tabclasses[0] = 0;
      tabclasses[1] = tabclasses[0];
      break;
    case 1:
      warning("twoclassconfidencebagmeasurer: you have only one class [%g]", tabclasses[0]);
      tabclasses[1] = tabclasses[0];
      break;
    case 2:
      message("twoclassconfidencebagmeasurer: two classes detected [%g and %g]", tabclasses[0], tabclasses[1]);
      break;
  }

  reset();
}

void TwoClassConfidenceBagMeasurer::reset()
{
}

TwoClassConfidenceBagMeasurer::~TwoClassConfidenceBagMeasurer()
{
  free(measure);
}

