#include "WinLayer.h"

WinLayer::WinLayer(int n_inputs_, int n_outputs_, int n_hu_gater)
{
  n_inputs = n_inputs_;
  n_outputs = n_outputs_;

  lin_layer = new Linear(n_inputs, n_outputs);
  mlp = new SigmMLP(n_inputs, n_hu_gater, n_outputs, false);  
}

void WinLayer::init()
{
  lin_layer->init();
  mlp->init();

  GradientMachine::init();
}

void WinLayer::allocateMemory()
{
  n_params = numberOfParams();

  addToList(&params, mlp->params);
  addToList(&params, lin_layer->params);

  addToList(&der_params, mlp->der_params);
  addToList(&der_params, lin_layer->der_params);

  alpha_mlp = (real *)xalloc(sizeof(real)*n_outputs);
  beta = (real *)xalloc(sizeof(real)*n_inputs);
  addToList(&outputs, n_outputs, (real *)xalloc(sizeof(real)*n_outputs));
}

void WinLayer::freeMemory()
{
  if(is_free)
    return;

  freeList(&params);
  freeList(&der_params);
  free(alpha_mlp);
  free(beta);
  freeList(&outputs, true);
  is_free = true;
}

void WinLayer::reset()
{
  lin_layer->reset();
  mlp->reset();
}

int WinLayer::numberOfParams()
{
  return(mlp->numberOfParams()+lin_layer->numberOfParams());
}

void WinLayer::forward(List *inputs)
{
  mlp->forward(inputs);
  lin_layer->forward(inputs);

  real *ptr_mlp = (real *)mlp->outputs->ptr;
  real *ptr_lin = (real *)lin_layer->outputs->ptr;
  real *ptr_out = (real *)outputs->ptr;

  for(int i = 0; i < n_outputs; i++)
    *ptr_out++ = *ptr_mlp++ * *ptr_lin++;
}

void WinLayer::backward(List *inputs, real *alpha)
{
  lin_layer->backward(inputs, alpha);

  real *beta_ptr = beta;
  real *beta_lin_ptr = lin_layer->beta;
  for(int i = 0; i < n_inputs; i++)
    *beta_ptr++ = *beta_lin_ptr++;

  real *der_ptr = (real *)lin_layer->der_params->ptr;
  for(int i = 0; i < n_outputs; i++)
  {
    real sum = 0;
    // Don't forget the bias...
    for(int j = 0; j < n_inputs+1; j++)
      sum += fabs(*der_ptr++);
    alpha_mlp[i] = sum;
  }

  real sum = 0;
  for(int i = 0; i < n_outputs; i++)
    sum += alpha_mlp[i];

  if(sum == 0)
    return;

  real *ptr_out = (real *)mlp->outputs->ptr;
//  print("[%g] ", sum);
  for(int i = 0; i < n_outputs; i++)
  {
//    print("%g ", 2./n_outputs-alpha_mlp[i]/sum);
    alpha_mlp[i] = 2.*(2./n_outputs - alpha_mlp[i]/sum - *ptr_out++);
  }
//  print("\n");

  mlp->backward(inputs, alpha_mlp);
}

void WinLayer::saveFILE(FILE *file)
{
  mlp->saveFILE(file);
  lin_layer->saveFILE(file);
}

void WinLayer::loadFILE(FILE *file)
{
  mlp->loadFILE(file);
  lin_layer->loadFILE(file);
}

WinLayer::~WinLayer()
{
}
