
#include "ks.h"

#define ONE_OVER_SQRT_2PI 0.39894228040143267794
#define SQRT2 1.4142135623730950


extern "C" int ks_compar_real(const void *p1, const void *p2)
{
  real v1 = *((real*)p1);
  real v2 = *((real*)p2);
  if (v1 > v2)
    return 1;
  if (v1 < v2)
    return -1;
  return 0;
}

real* ks_empirical_cumulative(int n, real* data, real& min, real& max, real* histo, int nhisto)
{
  // first sort the data
  qsort(data,n,sizeof(real),ks_compar_real);
  // set the min and max value, as well as the increment
  min = data[0];
  max = data[n-1];
  real inc = (max-min)/n;

  // then create the cumulative
  real* cumulative = (real*)xalloc(n*sizeof(real));
  real curr_max = min+inc;
  int k = 0;
  real previous = 0;
  int nbins = n/nhisto;
  if (nbins == 0)
    error("in ks_empirical_cumulative: n (%d) smaller than nhisto (%d)",n,nbins);
  for (int i=0;i<n;i++) {
    while (data[k] <= curr_max && k < n) {
      k++;
    }
    cumulative[i] = (real)k/n;
    if (i%nbins == 0) {
      histo[i] = cumulative[i] - previous;
      previous = cumulative[i];
    } else {
      histo[i] = histo[i-1];
    }
    curr_max += inc;
  }
  return cumulative;
}

real* ks_gaussian_cumulative(int n, real mu, real sigma, real min, real max)
{
  real* cumulative = (real*)xalloc(n*sizeof(real));
  real inc = (max - min)/n;
  real curr = min+inc/2;
  for (int i=0;i<n;i++,curr+=inc) {
    cumulative[i] = 0.5 * (1. + erf((curr - mu) / (SQRT2 * sigma)));
  }
  return cumulative;
}

real ks(int n,real* cumulative1,real* cumulative2, int precision)
{
  real k=0;
  real *pc1 = cumulative1;
  real *pc2 = cumulative2;
  for (int i=0;i<n;i++) {
    real diff = fabs(*pc1++ - *pc2++);
    k = diff > k ? diff : k;
  }
  k *= sqrt(n);

  real res = 0.0;
  real ks2 = k*k;
  for (int i=1;i<=precision;i++) {
     real x = ((i % 2) ? 1 : -1) * exp( -2 * ks2 * i * i );
     if (i==precision)
       res += 0.5*x;
     else
       res += x;
  }
  return 2 * res;
}

