#ifndef EMTRAINER_INC
#define EMTRAINER_INC

#include "Trainer.h"
#include "Distribution.h"
#include "SeqDataSet.h"

/** This class is used to train any distribution using the EM algorithm.
    
    @author Samy Bengio (bengio@idiap.ch)
*/
class EMTrainer : public Trainer
{
  public:
    /// the distribution to train
    Distribution *distribution;
    /// the training set is a SeqDataSet, since we are working with distributions
    SeqDataSet *sdata;
    /// the stopping criterion regarding the accuracy for EM
    real end_accuracy;
    /// the stopping criterion regarding the number of iterations for EM
    int max_iter;

    ///
    EMTrainer(Distribution *distribution_, SeqDataSet *data_);

    virtual void train(List *measurers);
    virtual void test(List *measurers);

    /** this method computes the most likely path into the distribution.
        mainly used for sequential distribution such as HMMs.
    */
    virtual void decode(List *measurers);

    virtual ~EMTrainer();
};

#endif
