#ifndef LOG_RBF_INC
#define LOG_RBF_INC

#include "GradientMachine.h"
#include "EMTrainer.h"

/** LogRBF layer for #GradientMachine#.
    Formally speaking, $ouputs[i] = -0.5 \sum_j gamma_ij^2 * (inputs[j] - mu_ij)^2$.\\
    $mu_ij$ and $gamma_ij$ are in #params#, with the following structure:\\
    $mu_00... mu_0n, gamma_00.. gamma_0n,..., $\\

    For a better initialization, one can provide a #EMTrainer# using a
    #Kmeans# distribution that will be used to initialize the means and
    gamma.

    @author Samy Bengio (bengio@idiap.ch)
*/
class LogRBF : public GradientMachine
{
  public:

    /// optional initialization using a Kmeans
    EMTrainer* initial_kmeans_trainer;

    ///
    LogRBF(int n_inputs_, int n_outputs_, EMTrainer* kmeans_trainer=NULL);

    //-----

    virtual int numberOfParams();
    virtual void init();
    virtual void reset();
    virtual void forward(List *inputs);
    virtual void backward(List *inputs, real *alpha);
    virtual ~LogRBF();
};

#endif
