#ifndef MAT_SEQ_DATASET_INC
#define MAT_SEQ_DATASET_INC

#include "SeqDataSet.h"
#include "StdDataSet.h"


/** This class enable to read sequences (hence inherits from #SeqDataSet#)
    from the classical Torch data format. Depending on the options, it
    can read one sequence per line (thus having sequences of size 1)
    or one sequence per file (then having sequences of size the size of
    the files). One can specify the number of inputs, observations and
    targets in the given file.
   
   @author Bison Ravi (francois.belisle@idiap.ch)
   @author Samy Bengio (samy.bengio@idiap.ch)
 */
class MatSeqDataSet : public SeqDataSet
{
  public :

    /** this boolean is used to know if the data has already been read
        by a #StdDataSet# and hence do not need to be freed at the end.
    */
    bool examples_from_stddataset;

    MatSeqDataSet( char* file, int n_inputs_, int n_observations_, int n_targets_,  
                   bool bin=false,  int max_load=-1 );
    MatSeqDataSet( char** files, int n_files, int n_inputs_, int n_observations_, int n_targets_,
                   bool bin=false,  int max_load=-1 );
    
    ///puts a data set in the Sequence framework
    MatSeqDataSet( StdDataSet* data,int n_inputs_,int n_observations_);

    virtual ~MatSeqDataSet();


    /** the following method is inherited from #SeqDataSet# */
    
    /** destroys data allocated by #createData# and the constructor 
     */
    virtual void freeMemory();

    /** reads the data into the #all_inputs# structure which
     * will be transfered in the #examples# structure in 
     * #createData#
     */
    void readMat( char* file, bool bin=false, int max_load=-1 );
    
    /** finds the number of lines in all of the given files */
    int lines( char* , bool bin );
    int lines( char**, int n_files, bool bin);
    
    /** finds the number of columns in a file */
    int columns( char* , bool bin );
};

#endif     
