#ifndef MULTINOMIAL_INC
#define MULTINOMIAL_INC

#include "Distribution.h"

/** This class can be used to model Multinomial Distributions.
    They can be trained using either EM (with EMTrainer) or gradient descent
    (with GMTrainer).

    @author Samy Bengio (bengio@idiap.ch)
*/
class Multinomial : public Distribution
{
  public:
    /// the number of different values that can take this discrete distribution
    int n_values;
    /// the prior weight given to each value. kind of smoother
    real prior_weights;

    /// optional initialization parameters
    List* initial_params;
    /// optional initialization file
    char* initial_file;

    /// the pointers to the parameters
    real* log_weights;

    /// the pointers to the d_parameters
    real* dlog_weights;

    /// accumulators for EM
    real*  weights_acc;

    Multinomial(int n_values_,real prior_weights_);

    virtual void reset();
    virtual int numberOfParams();
    virtual void allocateMemory();
    virtual void freeMemory();

    virtual void eMIterInitialize();
    virtual void iterInitialize();

    virtual real frameLogProbability(real *observations, real *inputs, int t);

    virtual void sequenceInitialize(List* inputs);
    virtual void eMSequenceInitialize(List* inputs);
    virtual void frameEMAccPosteriors(real *observations, real log_posterior, real *inputs, int t);
    virtual void eMUpdate();

    virtual void frameBackward(real *observations, real *alpha, real *inputs, int t);
    virtual void frameExpectation(real *observations, real *inputs, int t);

    virtual ~Multinomial();
};

#endif
