#ifndef NLL_MEASURER_INC
#define NLL_MEASURER_INC

#include "Measurer.h"
#include "SeqDataSet.h"

/** This class measures the negative log likelihood. In fact, it supposes
    that the input given is already the negative log likelihood. It is
    then normalized by the total number of frames.

    @author Samy Bengio (bengio@idiap.ch)
*/
class NllMeasurer : public Measurer
{
  public:
    /// contains the average negative log likelihood
    real *erreur;
    /// contains the total negative log likelihood
    real *erreur_c;
    /// contains a pointer to the negative log likelihood to measure
    List *inputs;
    /// local counter of the total number of frames
    int tot_n_frames;
    /// local pointer to the dataset, used to count the number of frames
    SeqDataSet *sdata;

    ///
    NllMeasurer(List *inputs_, SeqDataSet* data_, FILE *file_);
    ///
    NllMeasurer(List *inputs_, SeqDataSet* data_, const char *filename);

    virtual void init();
    virtual void reset();
    virtual void measureEx();
    virtual void measureIter();

    virtual ~NllMeasurer();
};

#endif
