#ifndef PARZEN_DISTRIBUTION_INC
#define PARZEN_DISTRIBUTION_INC

#include "Distribution.h"

/** This class can be used to model a Parzen density estimator with
    a Gaussian kernel

    $ p(x) = \frac{1}{N}\sum_i \frac{1}{(2 \Pi var)^{d/2}} \exp(- \frac{||x - x_i||^2}{2 var})$

    where the sum is done on the whole training set.

    @author Samy Bengio (bengio@idiap.ch)
*/
class ParzenDistribution : public Distribution
{
  public:
    /// the variance used
    real var;

    /// the dataset
    SeqDataSet* data;

    /// the indices of the training examples
    int *real_examples;
    int n_real_examples;

    /** in order to faster the computation, we can do some "pre-computation"
        pre-computed sum_log_var + n_obs * log_2_pi
    */
    real sum_log_var_plus_n_obs_log_2_pi;
    /// pre-computed -0.5 / var
    real minus_half_over_var;

    ParzenDistribution(SeqDataSet* data_, real var_);

    virtual void reset();
    virtual int numberOfParams();
    virtual void allocateMemory();
    virtual void freeMemory();

    virtual void setVar(real var_);

    virtual real frameLogProbability(real *observations, real *inputs, int t);
    virtual real frameLogProbabilityOneFrame(real *observations, real *mean);

    virtual void eMSequenceInitialize(List* inputs);
    virtual void sequenceInitialize(List* inputs);

    virtual void frameExpectation(real *observations, real *inputs, int t);

    virtual ~ParzenDistribution();
};

#endif
