#ifndef LOG_SIGMOID_INC
#define LOG_SIGMOID_INC

#include "GradientMachine.h"

/** Log-sigmoid layer for #GradientMachine#.
    The number of inputs/outputs is the number
    of units of this machine.
    Formally speaking, $ouputs[i] = log(1/(1+exp(-inputs[i])))$

    @author Ronan Collobert (collober@iro.umontreal.ca)
*/
class LogSigmoid : public GradientMachine
{
  public:
    real *buff;

    //-----
    
    /// Create a layer with <n_units> units.
    LogSigmoid(int n_units);

    //-----

    virtual void allocateMemory();
    virtual void freeMemory();
    virtual int numberOfParams();
    virtual void forward(List *inputs);
    virtual void backward(List *inputs, real *alpha);

    virtual ~LogSigmoid();
};

#endif
