#ifndef MLP_INC
#define MLP_INC

#include "ConnectedMachine.h"
#include "Linear.h"
#include "Tanh.h"
#include "SumMachine.h"
#include "Softmax.h"
#include "Sigmoid.h"

/** This class is a simple interface to the #ConnectedMachine# class that
    ca be used to build the well-known Multi Layer Perceptron type of
    neural networks. It contains a layer of #Linear# followed by a layer
    of #Tanh#, followed by a layer of #Linear# and optionally a layer
    of #Tanh# for classification experiments. Optionally, it also contains
    a direct connection from the inputs to the linear layer.
    Last, it can have a Softmax layer if you specify it. 

   @author Samy Bengio (bengio@idiap.ch)
*/
class MLP : public ConnectedMachine
{
  public:
    /// the first #Linear# layer
    Linear *hidden_layer;
    /// the first #Tanh# layer
    Tanh *hidden_tanh_layer;
    /// the second #Linear# layer
    Linear *outputs_layer;
    /// the optional second #Tanh# layer
    Tanh *outputs_tanh_layer;
    /// the optional second #Softmax# layer
    Softmax *outputs_softmax_layer;
    /// the optional second #Sigmoid# layer
    Sigmoid *outputs_sigmoid_layer;
    /// the number of hidden units
    int n_hidden;
    /// the number of inputs
    int n_inputs;
    /// the number of outputs
    int n_outputs;
    /// if this is false, add a #Tanh# layer
    bool is_linear_outputs;
    /** if this is true, add a #Softmax# layer.
        Note that  #is_linear_outputs# will not be taken in account.
    */
    bool is_softmax_outputs;
    bool is_sigmoid_outputs;
    /// if this is true, add a direct connection from inputs to #Linear#
    bool inputs_to_outputs;
    /// the eventual weight_decay
    real weight_decay;
    /// the direct #Linear# layer
    Linear* add_layer;
    /// if #inputs_to_outputs# is true, we also need a #SumMachine#
    SumMachine* sum_layer;

    MLP(int n_inputs_, int n_hidden, int n_outputs_);
    virtual void init();
    virtual ~MLP();
};

#endif
