#ifndef MACHINE_INC
#define MACHINE_INC

#include "Object.h"
#include "Measurer.h"
#include "List.h"

/** #Object# which can compute some outputs,
    given some inputs.

    @author Ronan Collobert (collober@iro.umontreal.ca)
*/
class Machine : public Object
{
  public:
    /// Number of inputs for the machine
    int n_inputs;

    /// Number of outputs
    int n_outputs;

    /** The outputs of the machine.
        Usually, the list contains only one node.

        The integer in each node is the number
        of components in the pointer #ptr# of this node.
        
        The sum of these integers is #n_outputs#.
    */
    List *outputs;

    ///
    Machine();

    /** Initialize the machine.
        All machines must call the #init()#
        function of their parents.
        Note that the options \emph{must} be
        set before calling #init()#.
        You \emph{must} call #init()#.
        After this call, the machine must be
        ready to do its job.
    */
    virtual void init() = 0;

    /** Given #inputs#, update #outputs#.
        The integer in each node of #inputs # is the number
        of components in the pointer #ptr# of this node.
        The sum of these integers is #n_inputs#.
    */
    virtual void forward(List *inputs) = 0;

    /// Reset the machine.
    virtual void reset() = 0;

    virtual ~Machine();
};

#endif

