#ifndef PARZEN_MACHINE_INC
#define PARZEN_MACHINE_INC

#include "Machine.h"
#include "DataSet.h"

/** This machine implements the Parzen Window estimator.
    Given a dataset (in the constructor), the #forward# method returns
    for a given input the average of the outputs of the training set
    weighted by a Gaussian kernel distance:

    $ y(x) = \frac{\sum_i t_i \exp(- \frac{||x - x_i||^2}{2 var})}{\sum_i \exp(- \frac{||x - x_i||^2}{2 var})}$

    The only parameter #var# is given in the constructor.

    @author Samy Bengio (bengio@idiap.ch)
*/
class ParzenMachine : public Machine
{
  public:

    /// the variance used
    real var;

    /// The dataset that contains the training set
    DataSet* data;

    /// the indices of the training examples
    int *real_examples;
    int n_real_examples;

    /// keep the denominator
    real denominator;

    ///
    ParzenMachine(DataSet* data_,real var_);

    virtual void forward(List *inputs);
    virtual void init();
    virtual void reset();

    /// change the value of var
    virtual void setVar(real var_);

    virtual ~ParzenMachine();
};

#endif
