#ifndef SOFTMAX_INC
#define SOFTMAX_INC

#include "GradientMachine.h"

/** Softmax layer for #GradientMachine#.

    The number of inputs/outputs is the number
    of units in the machine.

    Formally speaking, $ouputs[i] = 1/a*exp(inputs[i]-shift)$
    where $a = \sum_j exp(inputs[j]-shift)$.

    Options:
    \begin{tabular}{lcll}
      "shift"           & real &   shift to avoid overflow         & [0]\\
      "compute shift"   & bool &   compute shift to avoid overflow & [false]
    \end{tabular}

    (you can have the "shift" you want, if you want, or
    you can automaticaly compute the shift)

    @author Ronan Collobert (collober@iro.umontreal.ca)
*/
class Softmax : public GradientMachine
{
  public:
    real shift;
    bool calc_shift;

    //-----

    /// Create a Sigmoid layer with #n_units# units.
    Softmax(int n_units);

    //-----

    virtual int numberOfParams();
    virtual void forward(List *inputs);
    virtual void backward(List *inputs, real *alpha);

    virtual ~Softmax();
};

#endif
