// 1) Virer la normalisation: inutile en sparse
// 2) Corriger bug de liberation de memoire dans IOTorch

#ifndef SPARSE_DATA_SET_INC
#define SPARSE_DATA_SET_INC

#include "EuclideanDataSet.h"
#include "normalize.h"

/** Sparse Data Set.

    DataSet with a vector of sreals for inputs,
    and a vector of real for targets.
    
    This DataSet support normalization with mean/variance.

    Options:
    \begin{tabular}{lcll}
      "normalize inputs"  &  bool &   normalize the inputs by mean/stdv  & [false]\\
      "normalize targets" &  bool &   normalize the targets by mean/stdv & [false]
    \end{tabular}

    @author Ronan Collobert (collober@iro.umontreal.ca)
*/
class SparseDataSet : public EuclideanDataSet
{
  public:
    /// The inputs table.
    sreal **all_inputs;

    /// The targets table.
    real **all_targets;

    bool norm_inputs;
    bool norm_targets;
    real *mean_i;
    real *mean_o;
    real *stdv_i;
    real *stdv_o;

    //-----

    ///
    SparseDataSet();

    /** Normalize the #SparseDataSet# using the normalization values
        of another #SparseDataSet#. If the other dataset is not normalize,
        nothing will be done.

        Call it AFTER #init()#.
    */
    void normalizeUsingDataSet(SparseDataSet *data_norm);

    /** Set the inputs and the targets to a user tabular
        #n_dim# is the dimension of inputs vectors
        #n_targets_# is the dimension of targets vectors.

        Call them BEFORE #init()#.
    */
    void setInputs(sreal **data_, int n_dim, int n_examples_);
    ///
    void setTargets(real **y_, int n_targets_);

    /// Load the normalization parameters
    virtual void loadFILE(FILE *file);

    /// Save the normalization parameters
    virtual void saveFILE(FILE *file);

    //-----

    virtual void setRealExample(int t);
    virtual real realRealDotProduct(int i, int j);
    virtual real realDotProduct(int i, List *y);
    virtual real dotProduct(List *x, List *y);

    //-----

    virtual void init();
    void normalize();

    virtual ~SparseDataSet();
};

#endif
