#ifndef WEIGHTED_MSE_CRITERION_INC
#define WEIGHTED_MSE_CRITERION_INC

#include "Criterion.h"

/** Similar to #MseCriterion#, but you
    can put a weight on each example.

    @author Ronan Collobert (collober@iro.umontreal.ca)
*/
class WeightedMseCriterion : public Criterion
{
  public:

    /** Pointer on the weights.
        You can modify the weights when you want.
        
        The length of #weights# \emph{is} the
        #n_real_examples# of #data#.
    */
    real *weights;
    bool weights_is_allocated;

    //-----

    /** With this constructor, the weights are allocated
        and initializated to 1.
    */
    WeightedMseCriterion(DataSet *data_);

    /** With this constructor, you provide the #weights#.

       The length of #weights# \emph{must be} the
       #n_real_examples# of #data#.       
    */
    WeightedMseCriterion(DataSet *data_, real *weights_);

    //-----

    virtual void setDataSet(DataSet *data_);
    virtual void allocateMemory();
    virtual void freeMemory();
    virtual void forward(List *inputs);
    virtual void backward(List *inputs, real *alpha);
    virtual ~WeightedMseCriterion();
};

#endif
