#ifndef RANDOM_INC
#define RANDOM_INC

#include "general.h"

class ClassFormat;
class DataSet;

/** Random functions.
    
    @author Samy Bengio (bengio@idiap.ch)
    @author Yoshua Bengio (bengioy@iro.umontreal.ca)
    @author Ronan Collobert (collober@iro.umontreal.ca)
*/
//@{

/*  
    Special functions.
    -----------------
*/

/// Returns the natural logarithm of the gamma function.
real  gammln(real xx);
/// Returns in #indices# #n_indices# shuffled. (between 0 and #n_indices#).
void getShuffledIndices(int *indices, int n_indices);
/// Shuffle tabular, which contains #n_elems# of size #size_elem#.
void shuffle(void *tabular, int size_elem, int n_elems);

void getClassShuffledIndices(int *indices, int n_indices, ClassFormat* class_format, DataSet* data, int k_fold);

/*   
     Utilities for random numbers generation. 
     ---------------------------------------
*/

/// Initializes the random number generator with the cpu time.
void  seed();
/// Initialzes the random number generator with the given long "x".
void  manual_seed(long x);
/// Returns the current seed used by the random number generator.
long  get_seed();
/// Returns a random number uniformly distributed between 0 and 1.
real  uniform();
/// Returns a random number uniformly distributed between a and b.
real  bounded_uniform(real a,real b);
/// Returns an exponential distributed random number.
real  expdev();
/// Returns a random number gaussian with mean 0 and standard deviation 1.
real  gaussian_01();
/// Returns a random number gaussian with mean "mu" and standard dev "sigma".
real  gaussian_mu_sigma(real mu, real sigma);
/// Returns a gamma distributed random number.
real  gamdev(int ia);
/// Returns a poisson random number with lambda = "xm".
real  poidev(real xm);
/// Returns a binomial random number with probability = 'pp' and trials number = 'n'.
real  bnldev(real pp, int n);

//@}

#endif


