#ifndef IO_TORCH_INC
#define IO_TORCH_INC

#include "general.h"
#include "Object.h"

//-----

typedef struct BlkMem_
{
    real *internal_alloc;
    real **data;
    real **targets;
    int n_examples;
    bool data_is_internal;
    bool targets_is_internal;
    int size_data;
    int size_targets;
} BlkMem;

//-----

/** Load and save file in the Torch format.
    @author Ronan Collobert (collober@iro.umontreal.ca)
 */
class IOTorch : public Object
{
  public:
    
    BlkMem *blocks;
    int n_blocks;

    ///
    IOTorch();

    int addBlkMem(real *internal_alloc, real **data, real **targets, int n_examples, bool data_is_internal, bool targets_is_internal, int size_data, int size_targets);
    void destroyBlkMem(int number);

    //-----

    /** Load the file #file#.
        Each example must have #n_inputs# inputs and #n_targets# targets.
        #bin# is #true# if the file is in binary format.
        If #max_load# > 0, load only the first #max_load# examples.
        Returns: \begin{itemize}
          \item #data_# which contains all inputs (the memory allocation is done by this function)
          \item #y_# which contains all targets (the memory allocation is done by this function)
          \item #n_examples# which contains the real number of loaded examples.
        \end{itemize}
    */
    int loadData(const char  *file,  real ***data_, real ***y_, int n_inputs, int n_targets, int *n_examples, bool bin=false, int max_load=-1);

    /// Sparse version of the previous function
    int loadData(const char  *file, sreal ***data_, real ***y_, int n_inputs, int n_targets, int *n_examples, bool bin=false, int max_load=-1);

    /// Similar function, but to save data
    void saveData(const char *file,  real **data, real **y, int l, int n_inputs, int n_targets, bool bin=false, int max_save=-1);

    /// Similar function, but to save sparse data
    void saveData(const char *file, sreal **data, real **y, int l, int n_inputs, int n_targets, bool bin=false, int max_save=-1);

    //-----

    ~IOTorch();
};

#endif
