#include "Kernel.h"

Kernel::Kernel(DataSet *data_)
{
  data = data_;
}

void Kernel::init()
{
}

void Kernel::reset()
{
}


Kernel::~Kernel()
{
}

//=====

DotKernel::DotKernel(EuclideanDataSet *edata_) : Kernel(edata_)
{
  edata = edata_;
}

real DotKernel::eval(int i, int j)
{
  return(edata->dotProduct(i, j));
}

real DotKernel::realEval(int i, List *y)
{
  return(edata->realDotProduct(i, y));
}

DotKernel::~DotKernel()
{
}

//=====

PolynomialKernel::PolynomialKernel(EuclideanDataSet *edata_) : Kernel(edata_)
{
  edata = edata_;
  d = 2;
  s = 1;
  r = 1;
  addOption("degree", sizeof(int), &d, &d, "degree");
  addOption("mul cst", sizeof(real), &s, &s, "multiplicative constant");
  addOption("add cst", sizeof(real), &r, &r, "additive constant");
}

real PolynomialKernel::eval(int i, int j)
{
  real z = s*edata->dotProduct(i, j) + r;

  // la fonction pow rame a donf
  for(int t = 1; t < d; t++)
    z *= z;
    
  return(z);
}

real PolynomialKernel::realEval(int i, List *y)
{
  real z = s*edata->realDotProduct(i, y) + r;

  // la fonction pow rame a donf
  for(int t = 1; t < d; t++)
    z *= z;
    
  return(z);
}

PolynomialKernel::~PolynomialKernel()
{
}

//=====

GaussianKernel::GaussianKernel(EuclideanDataSet *edata_) : Kernel(edata_)
{
  edata = edata_;
  precalc = NULL;
  precalc_real = NULL;
  g = 1./(10.*10.);
  addOption("gamma", sizeof(real), &g, &g, "gamma");
}

void GaussianKernel::reset()
{
  precalc      = (real *)xrealloc(precalc,      sizeof(real)*data->n_examples);
  precalc_real = (real *)xrealloc(precalc_real, sizeof(real)*data->n_real_examples);
  message("GaussianKernel: precalculating...");
  for(int i = 0; i < data->n_examples; i++)
    precalc[i] = edata->dotProduct(i, i);
  for(int i = 0; i < data->n_real_examples; i++)
    precalc_real[i] = edata->realRealDotProduct(i, i);
  message("...ok");
}

real GaussianKernel::eval(int i, int j)
{    
  return(exp(g*( 2.*edata->dotProduct(i,j)-precalc[i]-precalc[j] )));
}

real GaussianKernel::realEval(int i, List *y)
{
  return(exp(g*( 2.*edata->realDotProduct(i,y)-precalc_real[i]-edata->dotProduct(y,y) )));
}

GaussianKernel::~GaussianKernel()
{
  free(precalc);
  free(precalc_real);
}

//=====

SigmoidKernel::SigmoidKernel(EuclideanDataSet *edata_) : Kernel(edata_)
{
  edata = edata_;
  s = 1;
  r = 1;
  addOption("mul cst", sizeof(real), &s, &s, "multiplicative constant");
  addOption("add cst", sizeof(real), &r, &r, "additive constant");
}

real SigmoidKernel::eval(int i, int j)
{
  return(tanh(s*edata->dotProduct(i, j) + r));
}

real SigmoidKernel::realEval(int i, List *y)
{
  return(tanh(s*edata->realDotProduct(i, y) + r));
}

SigmoidKernel::~SigmoidKernel()
{
}

//=====
