#include "LogSigmoid.h"

LogSigmoid::LogSigmoid(int n_units)
{
  n_outputs = n_units;
  buff = NULL;
}

int LogSigmoid::numberOfParams()
{
  return(0);
}

void LogSigmoid::allocateMemory()
{
  GradientMachine::allocateMemory();
  buff = (real *)xalloc(sizeof(real)*n_outputs);
}

void LogSigmoid::freeMemory()
{
  if(is_free)
    return;

  GradientMachine::freeMemory();
  free(buff);

  is_free = true;
}

void LogSigmoid::forward(List *inputs)
{
  real *ptr_out = (real *)outputs->ptr;
  real *ptr_buff = buff;
  while(inputs)
  {
    real *x = (real *)inputs->ptr;
    for(int j = 0; j < inputs->n; j++)
    {
      *ptr_buff = exp(- *x++);
      *ptr_out++ = -log(1.+ *ptr_buff++);
    }
    inputs = inputs->next;
  }
}

void LogSigmoid::backward(List *inputs, real *alpha)
{
  real *alpha_ptr = alpha;
  real *beta_ptr = beta;
  real *buff_ptr = buff;
  for(int i = 0; i < n_outputs; i++)
  {
    real z = *buff_ptr++;
    *beta_ptr++ = *alpha_ptr++ * z / (1. + z);
  }
}

LogSigmoid::~LogSigmoid()
{
  freeMemory();
}
