#include "Mixer.h"

Mixer::Mixer(int n_experts_, int n_outputs_per_expert)
{
  n_experts = n_experts_;
  n_inputs = n_experts*(n_outputs_per_expert+1);
  n_outputs = n_outputs_per_expert;
}

int Mixer::numberOfParams()
{
  return(0);
}

void Mixer::forward(List *inputs)
{
  real *ptr_out = (real *)outputs->ptr;
  for(int j = 0; j < n_outputs; j++)
    *ptr_out++ = 0;
  
  real *x = (real *)inputs->ptr;
  List *out_experts = inputs->next;
  int k = out_experts->n;
  real *y = (real *)out_experts->ptr;
  for(int i = 0; i < n_experts; i++)
  {
    real *ptr_out = (real *)outputs->ptr;
    for(int j = 0; j < n_outputs; j++)
    {
      if(k == 0)
      {
        out_experts = out_experts->next;
        k = out_experts->n;
        y = (real *)out_experts->ptr;        
      }
      *ptr_out++ += *x * *y++;
      k--;
    }
    x++;
  }
}

void Mixer::backward(List *inputs, real *alpha)
{
  real *ptr_beta = beta;
  for(int i = 0; i < n_experts; i++)
    *ptr_beta++ = 0;
    
  ptr_beta = beta;
  List *out_experts = inputs->next;
  int k = out_experts->n;
  real *y = (real *)out_experts->ptr;
  for(int i = 0; i < n_experts; i++)
  {
    real *ptr_alpha = alpha;
    for(int j = 0; j < n_outputs; j++)
    {
      if(k == 0)
      {
        out_experts = out_experts->next;
        k = out_experts->n;
        y = (real *)out_experts->ptr;        
      }
      *ptr_beta += *ptr_alpha++ * *y++;
      k--;
    }
    ptr_beta++;
  }

  real *x = (real *)inputs->ptr;
  for(int i = 0; i < n_experts; i++)
  {
    real *ptr_alpha = alpha;
    for(int j = 0; j < n_outputs; j++)
      *ptr_beta++ = *ptr_alpha++ * *x;

    x++;
  }
}

Mixer::~Mixer()
{
}
