#ifndef MIXER_INC
#define MIXER_INC

#include "GradientMachine.h"

/** Mixer useful for experts mixtures.
    Formally speaking, it computes:
    $outputs[i] = \sum_j a_j * inputs_j[i]$

    where
    \begin{itemize}
      \item ${a_1,...,a_n}$ are in the table
      of the first node of the #inputs# list,
      when you call #forward()#.
      
      \item $inputs_j$ are the inputs of the j-th expert.
      Therefore, the #inputs# list has the structure
      ${a, inputs_1, inputs_2, ...}$.
      Only $a$ must be alone in one node.
    \end{itemize}

    @author Ronan Collobert (collober@iro.umontreal.ca)
*/
class Mixer : public GradientMachine
{
  public:
    int n_experts;

    //-----

    ///
    Mixer(int n_inputs_, int n_outputs_per_expert);

    //-----

    virtual int numberOfParams();
    virtual void forward(List *inputs);
    virtual void backward(List *inputs, real *alpha);

    virtual ~Mixer();
};

#endif
