#ifndef OBJECT_INC
#define OBJECT_INC

#include "general.h"
#include "random.h"

//-----

typedef struct Option_
{
    char *name;
    char *help;
    int size;
    void *ptr;
} Option;

//-----

/** Provides a useful interface for managing options.
    Class inherited by almost all classes in Torch.
    
    @author Ronan Collobert (collober@iro.umontreal.ca)
*/
class Object
{
  public:

    int n_options;
    Option *options;

    ///
    Object();
    
    //-----

    /** Add the option #name#.
        This one has a pointer on #ptr# and has the size #size#.
        The value used for initialization is contained at the adress #initvalue#.
        You can provide an help in #help#.
        (Note that if #help# is empty, its an empty string and not NULL).
    */
    void addOption(const char *name, int size, void *ptr, void *initvalue=NULL, const char *help="");

    /// Set the option #name# to the value contained at #ptr#.
    void setOption(const char *name, void *ptr);

    /** Several wrappers of #setOption()# for
        Int, Real and Bool options.
        Be carefull: there is no type checking.
    */
    void setIOption(const char *name, int option);

    ///
    void setROption(const char *name, real option);

    ///
    void setBOption(const char *name, bool option);

    /// Load the object from a file pointer (\emph{not the options})
    virtual void loadFILE(FILE *file);

    /// Save the object to a file pointer (\emph{not the options})
    virtual void saveFILE(FILE *file);

    /// Load the machine from a file name (\emph{not the options})
    void load(const char *filename);

    /// Save the machine to a file name (\emph{not the options})
    void save(const char *filename);

    //-----

    virtual ~Object();
};

#endif
