#include "OneHotClassFormat.h"

OneHotClassFormat::OneHotClassFormat(DataSet *data)
{
  n_classes = data->n_targets;
  class_labels_buffer = (real *)xalloc(sizeof(real)*n_classes*n_classes);
  class_labels = (real **)xalloc(sizeof(real *)*n_classes);
  for(int i = 0; i < n_classes; i++)
  {
    class_labels[i] = class_labels_buffer+i*n_classes;
    for(int j = 0; j < n_classes; j++)
      class_labels[i][j] = 0.;
    class_labels[i][i] = 1.;
  }
}

OneHotClassFormat::OneHotClassFormat(int n_targets)
{
  n_classes = n_targets;
  class_labels_buffer = (real *)xalloc(sizeof(real)*n_classes*n_classes);
  class_labels = (real **)xalloc(sizeof(real *)*n_classes);
  for(int i = 0; i < n_classes; i++)
  {
    class_labels[i] = class_labels_buffer+i*n_classes;
    for(int j = 0; j < n_classes; j++)
      class_labels[i][j] = 0.;
    class_labels[i][i] = 1.;
  }
}

int OneHotClassFormat::getOutputSize()
{
  return n_classes;
}

void OneHotClassFormat::fromOneHot(List *outputs, List *one_hot_outputs)
{
  copyList(outputs,one_hot_outputs);
}

void OneHotClassFormat::toOneHot(List *outputs, List *one_hot_outputs)
{
  copyList(one_hot_outputs,outputs);
}

int OneHotClassFormat::getTargetClass(void *target)
{
  real *out = (real *)target;

  real z = *out++;
  int index = 0;
  for(int i = 1; i < n_classes; i++)
  {
    if(*out > z)
    {
      index = i;
      z  = *out;
    }
    out++;
  }
  
  return(index);
}

int OneHotClassFormat::getOutputClass(List *outputs)
{
  real z = *((real *)outputs->ptr);
  int index = 0;

  int i = 0;
  while(outputs)
  {
    real *out = (real *)outputs->ptr;
    for(int j = 0; j < outputs->n; j++)
    {
      if(*out > z)
      {
        index = i;
        z = *out;
      }
      out++;
      i++;
    }
    outputs = outputs->next;
  }
  
  return(index);
}

OneHotClassFormat::~OneHotClassFormat()
{
  free(class_labels_buffer);
  free(class_labels);
}
