#ifndef OPTIMIZER_INC
#define OPTIMIZER_INC

#include "Object.h"
#include "GradientMachine.h"
#include "DataSet.h"
#include "Criterion.h"

/** Optimizer for the #GMTrainer# class.

    Given a #criterion#, a #machine# (#GradientMachine#)
    and a #DataSet# #data#, it optimizes the #machine#
    with respect to the #criterion# on the #data#.

    @author Ronan Collobert (collober@iro.umontreal.ca)
    @see GMTrainer
*/
class Optimizer : public Object
{
  public:
    GradientMachine *machine;
    DataSet *data;
    Criterion *criterion;

    //-----

    ///
    Optimizer();

    /** Optimize.
        The measurers #measurers# will be called
        during the optimization.
    */
    virtual void train(List *measurers) = 0;

    /** Set the #machine#.
        This method could be called at every time
        before a call to #train()#.
    */
    void setMachine(GradientMachine *machine_);

    /** Set the #data#.
        This method could be called at every time
        before a call to #train()#.
    */
    void setDataSet(DataSet *data_);

    /** Set the #criterion#.
        This method could be called at every time
        before a call to #train()#.
    */
    void setCriterion(Criterion *criterion_);

    //-----

    virtual ~Optimizer();
};

#endif
