#ifndef QC_CACHE_INC
#define QC_CACHE_INC

#include "Object.h"

/** "Cache" used by the Quadratic Constrained Trainer (#QCTrainer#).
    This class provides the Q (symetric) matrix of the
    quadratic problem.

    @author Ronan Collobert (collober@iro.umontreal.ca)
    @see QCTrainer
*/
class QCCache : public Object
{
  public:

    //-----

    ///
    QCCache();

    /// Returns the adress of the row/column #index# for the Q matrix.
    virtual real *adressCache(int index) = 0;

    /// Allocate the cache.
    virtual void allocate() = 0;

    /// Erase the cache (but don't destroy it).
    virtual void clear() = 0;

    /// Destroy the cache.
    virtual void destroy() = 0;

    /** The index of the active variables will be those
        contained in #active_var#, and the number of active
        variable will be #*n_active_var_#.

        (It's usefull for some problems)
        
        @see QCMachine
        @see QCTrainer
    */
    virtual void setBoosterMode(int *n_active_var_, int *active_var_) = 0;

    //-----

    virtual ~QCCache();
};

#endif
