#ifndef SATURATION_MEASURER_INC
#define SATURATION_MEASURER_INC

#include "Measurer.h"
#include "GradientMachine.h"

/** Measure the saturation of a #GradientMachine#.

    Save in #file# (on two columns)
      - the absolute mean of the outputs of the machine.
      - the absolute mean of the derivative of the parameters of the machine.

   #measure# = {<[real] mean of outputs> <[real] mean of gradients>
                <[real] current sum of ouputs> <[real] current sum of gradients>}

   @author Ronan Collobert (collober@iro.umontreal.ca)
*/
class SaturationMeasurer : public Measurer
{
  public:
    real *sum_outputs;
    real *sum_outputs_c;
    real n_sum_out;
    real *sum_derivees;
    real *sum_derivees_c;
    real n_sum_der;

    GradientMachine *machine;

    //-----

    ///
    SaturationMeasurer(GradientMachine *machine_, DataSet *data_, FILE *file_);
    ///
    SaturationMeasurer(GradientMachine *machine_, DataSet *data_, const char *filename);

    //-----

    virtual void init();
    virtual void reset();
    virtual void measureEx();
    virtual void measureIter();

    virtual ~SaturationMeasurer();
};

#endif
