#include "Softmax.h"

Softmax::Softmax(int n_units)
{
  n_inputs = n_units;
  n_outputs = n_units;
  shift = 0;
  calc_shift = false;
  addOption("shift", sizeof(real), &shift, &shift, "shift to avoid overflow");
  addOption("compute shift", sizeof(bool), &calc_shift, &calc_shift, "compute shift to avoid overflow");
}

int Softmax::numberOfParams()
{
  return(0);
}

void Softmax::forward(List *inputs)
{
  if(calc_shift)
  {
    List *inputs_ = inputs;
    shift = ((real *)inputs_->ptr)[0];
    while(inputs_)
    {
      real *x = (real *)inputs_->ptr;
      for(int j = 0; j < inputs_->n; j++, x++)
      {
        if(*x > shift)
          shift = *x;
      }
      inputs_ = inputs_->next;
    }
  }
  
  real *ptr_out = (real *)outputs->ptr;
  real sum = 0;
  while(inputs)
  {
    real *x = (real *)inputs->ptr;
    for(int j = 0; j < inputs->n; j++)
    {
      real z = exp(*x++ - shift);
      *ptr_out++ = z;
      sum += z;
    }
    inputs = inputs->next;
  }

  ptr_out = (real *)outputs->ptr;
  for(int i = 0; i < n_outputs; i++)
    *ptr_out++ /= sum;
}

void Softmax::backward(List *inputs, real *alpha)
{
  real *ptr_alpha = alpha;
  real *ptr_outputs = (real *)outputs->ptr;
  real sum = 0;
  for(int i = 0; i < n_outputs; i++)
    sum += *ptr_alpha++ * *ptr_outputs++;

  ptr_alpha = alpha;
  ptr_outputs = (real *)outputs->ptr;
  real *ptr_beta = beta;
  for(int i = 0; i < n_outputs; i++)
    *ptr_beta++ = *ptr_outputs++ * (*ptr_alpha++ - sum);
}

Softmax::~Softmax()
{
}
