#ifndef STOCHASTIC_GRADIENT_INC
#define STOCHASTIC_GRADIENT_INC

#include "Optimizer.h"
#include "random.h"

/** Stochastic Gradient Optimizer for GMTrainer

    Options:
    \begin{tabular}{lcll}
      "end accuracy"         & real  &  end accuracy                   & [0.0001]\\
      "learning rate"        & real  &  learning rate                  & [0.01]\\
      "learning rate decay"  & real  &  learning rate decay            & [0]\\
      "max iter"             & int   &  maximum number of iterations   & [-1]
    \end{tabular}

    @see GMTrainer
    @author Ronan Collobert (collober@iro.umontreal.ca)
*/
class StochasticGradient : public Optimizer
{
  public:
    
    real learning_rate;
    real learning_rate_decay;
    real end_accuracy;
    int max_iter;

    //-----

    ///
    StochasticGradient();
    virtual void train(List *measurers);

    //-----

    virtual ~StochasticGradient();
};

#endif
