#include "normalize.h"

void MSTDVNormalize(real **tab, real *mean_buff, real *std_buff, int n_ex, int n_dim)
{
  for(int d = 0; d < n_dim; d++)
  {
    real std_buff_  = 0;
    real mean_buff_ = 0;
    for(int i = 0; i < n_ex; i++)
    {
      real z = tab[i][d];
      std_buff_  += z*z;
      mean_buff_ += z;
    }
    std_buff_  /= (real)n_ex;
    mean_buff_ /= (real)n_ex;
    std_buff_  -= mean_buff_*mean_buff_;
    if(std_buff_ <= 0)
    {
      warning("normalize: a column has a null stdv");
      std_buff_ = 1;
    }
    std_buff[d]  = sqrt(std_buff_);
    mean_buff[d] = mean_buff_;
  }
}

void MSTDVSparseNormalize(sreal **tab, real *mean_buff, real *std_buff, int n_ex, int n_dim)
{
  for(int d = 0; d < n_dim; d++)
  {
    mean_buff[d] = 0;
    std_buff[d] = 0;
  }

  for(int i = 0; i < n_ex; i++)
  {
    sreal *z = tab[i];
    while(z->index > 0)
    {
      real zz = z->value;
      int d = z->index;

      std_buff[d]  += zz*zz;
      mean_buff[d] += zz;

      z++;
    }
  }

  for(int d = 0; d < n_dim; d++)
  {
    std_buff[d]  /= (real)n_ex;
    mean_buff[d] /= (real)n_ex;
    std_buff[d]  -= mean_buff[d]*mean_buff[d];
    if(std_buff[d] <= 0)
    {
      warning("normalize: a column has a null stdv");
      std_buff[d] = 1;
    }
    else
      std_buff[d]  = sqrt(std_buff[d]);
  }
}
