/*
        rdpipe.cc
	
	Copyright (C) 2002 Sebastien Marcel (marcel@idiap.ch)

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "globalvideo.h"

using namespace GlobalVideo;

void InitPipe(char *qos_pipe_name, char *frame_pipe_name)
{
  	printf("QoS pipe name = <%s>\n", qos_pipe_name);
  	qosPipe = open(qos_pipe_name, O_RDWR);

  	printf("Frame pipe name = <%s>\n", frame_pipe_name);
  	framePipe = open(frame_pipe_name, O_RDWR);

  	framenumber = 0;
	
  	frameBuffer = (unsigned char *) malloc(4 * image_width * image_height * sizeof(unsigned char));
  	frameRgb = (short *) malloc(3 * image_width * image_height * sizeof(short));
  	frameReal = (float *) malloc(image_width * image_height * sizeof(float));
}

void ClosePipe()
{
  	close(framePipe);  
  	close(qosPipe);  

	free(frameReal);
	free(frameRgb);
	free(frameBuffer);
}

#define P_R 	0.299 
#define P_G 	0.587
#define P_B  	0.114

int GetNewFrame()
{
	// Image info
  	int width_;
  	int height_;
  	int framesize_;
  
	// Indexes 
  	int i, j, ShiftRGB;
  	int pix, iOffset = 0;
  	int gPlane, bPlane;
  
	// Buffer to store frame from the pipe 
  	int n_read = 0;

	//
  	printf("\nReading header from the pipe ...\n");
  	read(qosPipe, &videoheader, sizeof(VideoHeader));

	if(videoheader.width == 0)
	{
	   	printf("Error: width in header is zero\n");
		
		return false;
	}
	
	if(videoheader.height == 0)
	{
	   	printf("Error: height in header is zero\n");
		
		return false;
	}
	
	   
	if(videoheader.width != image_width)
	{
	   	printf("Error: width in header (%d) is not equal to %d\n", videoheader.width, image_width);
		
		return false;
	}
	
	   
	if(videoheader.height != image_height)
	{
	   	printf("Error: height in header (%d) is not equal to %d\n", videoheader.height, image_height);
		
		return false;
	}
	

  	if(videoheader.shot_end == 10) 
	{		
		printf("End of shot reached\n");
		
		return false;
	}

	//
  	width_ = videoheader.width;
  	height_ = videoheader.height;
  	framesize_ = videoheader.buffer_size;

	printf(" + width = %d\n", width_);
	printf(" + height = %d\n", height_);
	
	//
  	printf("Reading frame from the pipe ...\n");
  	n_read = read(framePipe, frameBuffer, framesize_);
  	printf(" + %6.6d bytes in buffer\n", n_read);
  	fflush (stdout);
  
  	// Convert the RGBX frame to three planes, one for Red, one for Green and one for Blue
  	ShiftRGB = 0;
  	gPlane = width_ * height_;
  	bPlane = 2 * gPlane;
  
	//
  	for (i = 0 ; i < height_ ; i++)
	{
		for (j = 0 ; j < width_ ; j++)
		{
			pix = iOffset + j;
			 
			frameReal[pix] = (float) P_R * frameBuffer[ShiftRGB + 2] + P_G * frameBuffer[ShiftRGB + 1] + P_B * frameBuffer[ShiftRGB];
			 
			frameRgb[pix] = frameBuffer[ShiftRGB + 2];		// R
			frameRgb[pix + gPlane] = frameBuffer[ShiftRGB + 1];	// G
			frameRgb[pix + bPlane] = frameBuffer[ShiftRGB];		// B
							
			ShiftRGB += 4;
		}
		
		iOffset += width_;
	}
  	// end of RGBX to 3-planes buffer conversion

	framenumber++;

	return true;
}


