/*
        video2ppm.cc
	
	Copyright (C) 2002 Sebastien Marcel (marcel@idiap.ch)

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "CmdLine.h"

const char *help = "This program read rgb frames from v2f pipe and save it as ppm";

#include "globalvideo.h"

void InitPipe(char *qos_pipe_name, char *frame_pipe_name);
void ClosePipe();
int GetNewFrame();
int SavePPM(char *filename, int w, int h, short *frame);

void InitClient();
void RunClient();
void CloseClient();

int main(int argc, char **argv)
{
   	// Command line parameters
   	char *qos_pipe_name;
   	char *frame_pipe_name;
	int max_image;
	CmdLine cmd;

	// Prepare the command line
	cmd.info(help);
	cmd.addText("\nArguments:");
	cmd.addSCmdArg("qos pipe", &qos_pipe_name,"the header pipe to process");
	cmd.addSCmdArg("frame pipe", &frame_pipe_name,"the frame pipe to process");
	cmd.addICmdArg("width", &GlobalVideo::image_width,"the expected image width");
	cmd.addICmdArg("height", &GlobalVideo::image_height,"the expected image height");
	cmd.addText("\nframe options:");
	cmd.addBCmdOption("-ppm", &GlobalVideo::bSavePPM, false, "Save individual images as ppm");  
	cmd.addICmdOption("-I", &max_image, -1, "maximum number of images to process");  
	// Read the command line
	cmd.read(argc, argv);

	// Filename to save the ppm image 
  	char filename_ppm[200];  
	bool bRead;
	
	// Init pipes
	InitPipe(qos_pipe_name, frame_pipe_name);  
	printf("Pipes initialized.\n");  

	// Init client specific 
	InitClient();
	printf("Client initialized.\n");  

	printf("Let's go now.\n\n");  

	// While there is a frame to read in the buffer
	bRead = GetNewFrame();
	while(bRead == true)
	{
		printf("\rtc = %6.6d width = %4.4d height = %4.4d shot_end = %1.1d", 
	      		GlobalVideo::videoheader.timecode,
			GlobalVideo::videoheader.width,
			GlobalVideo::videoheader.height,
			GlobalVideo::videoheader.shot_end);
		fflush(stdout);

		// The client is processing the current frame : frameRgb and frameReal
		RunClient();
		
		if(GlobalVideo::bSavePPM == true)
		{
  			sprintf(filename_ppm, "image%05d.ppm", GlobalVideo::framenumber);
		  	SavePPM(filename_ppm, GlobalVideo::image_width, GlobalVideo::image_height, GlobalVideo::frameRgb);  
		}

		if(max_image != -1)
		{
			if(GlobalVideo::framenumber >= max_image) bRead = false; 
			else bRead = GetNewFrame();
		}
		else bRead = GetNewFrame();

	} // end of while

	printf("\n\nEnd of processing.\n");  

	printf("Closing client.\n");  
	CloseClient();

	printf("\n\nEnd of processing. Closing pipes.\n");  
	ClosePipe();
	  
	return 1;
}
